#include "mainWindow.h"
#include <sstream>

MainWindow::MainWindow(QWidget *parent, const char *name) 
  : QMainWindow(parent, name)
{
  setCaption(name);

  workspace = new QWorkspace(this,"espace de travail");
  setCentralWidget(workspace);
 
  createActions();
  createMenus();
  createToolsBar();
  createStatusBar();
  
  axis = new Window2D(workspace,"axe");
  section = new Window2D(workspace,"section");
  profile = new Window2D(workspace,"profil");
  opengl = new Window3D(workspace,"visualisation 3D");
 
  showMaximized();
  workspace->tile();

  filename = "nouveau";
}

MainWindow::~MainWindow()
{
  delete axis;
  delete section;
  delete profile;
  delete opengl;

  delete workspace;

  delete actionSegment;
  delete actionCircle;
  delete actionBezier;
  delete actionBSpline;
  
  delete actionOpen;
  delete actionNew;
  delete actionSave;
  delete actionSaveAs;
  delete actionImportVRML;
  delete actionImportPGN;
  delete actionExportVRML;
  delete actionExportPGN;
  delete actionQuit;

  delete actionSimpleExtrusion;
  delete actionProfileExtrusion;

  delete actionTile;
  delete actionCascade;
  delete changeWin;
  delete clearWindow;

  delete actionChangeSample;

  delete subMenuImport;
  delete subMenuExport;
  delete menuPrimitives;
  delete menuExtrusion;
  delete menuDisplay;
  delete menuHelp;
  
  delete spinboxTwist;
  delete sampleComboBox;
  delete infos;
  
  delete barFile;
  delete barPrimitives;
  delete barDisplay;
  delete barTwist;
  delete barSample;

  delete timerSimpleExtrusion;
  delete timerProfileExtrusion;
}

void MainWindow::createActions()
{
  actionSegment = new QAction(tr("Segment"),Key_F1 ,this);
  actionSegment->setIconSet(QPixmap::fromMimeSource("src/interface/icones/segment.png"));
  actionSegment->setStatusTip(tr("Dessiner un segment"));
  connect(actionSegment,SIGNAL(activated()),this,SLOT(slotDrawSegment()));

  actionCircle = new QAction(tr("Cercle"),Key_F2 ,this);
  actionCircle->setIconSet(QPixmap::fromMimeSource("src/interface/icones/cercle.png"));
  actionCircle->setStatusTip(tr("Dessiner un cercle"));
  connect(actionCircle,SIGNAL(activated()),this,SLOT(slotDrawCircle()));
  
  actionBezier = new QAction(tr("Bzier"),Key_F3 ,this);
  actionBezier->setIconSet(QPixmap::fromMimeSource("src/interface/icones/bezier.png"));
  actionBezier->setStatusTip(tr("Dessiner une courbe de Bzier"));
  connect(actionBezier,SIGNAL(activated()),this,SLOT(slotDrawBezier()));
  
  actionBSpline = new QAction(tr("BSpline"),Key_F4 ,this);
  actionBSpline->setIconSet(QPixmap::fromMimeSource("src/interface/icones/bspline.png"));
  actionBSpline->setStatusTip(tr("Dessiner une BSpline"));
  connect(actionBSpline,SIGNAL(activated()),this,SLOT(slotDrawBSpline()));
   
  actionNew = new QAction(tr("Nouveau"),tr("Ctrl+N"),this);
  actionNew->setIconSet(QPixmap::fromMimeSource("src/interface/icones/nouveau.png"));
  actionNew->setStatusTip(tr("Creation d'un nouveau document"));
  connect(actionNew,SIGNAL(activated()),this,SLOT(slotNew()));
  
  actionOpen = new QAction(tr("Ouvrir"),tr("Ctrl+O"),this);
  actionOpen->setIconSet(QPixmap::fromMimeSource("src/interface/icones/ouvrir.png"));
  actionOpen->setStatusTip(tr("Ouverture d'un document existant"));
  connect(actionOpen,SIGNAL(activated()),this,SLOT(slotOpen()));
  
  actionSave = new QAction(tr("Enregistrer"),tr("Ctrl+S"),this);
  actionSave->setIconSet(QPixmap::fromMimeSource("src/interface/icones/sauver.png"));
  connect(actionSave,SIGNAL(activated()),this,SLOT(slotSave()));
  
  actionSaveAs = new QAction(tr("Enregistrer sous..."),tr(""),this);
  actionSaveAs->setIconSet(QPixmap::fromMimeSource("src/interface/icones/sauver.png"));
  connect(actionSaveAs,SIGNAL(activated()),this,SLOT(slotSaveAs()));
  
  actionQuit = new QAction(tr("Quitter"),tr("Ctrl+Q"),this);
  connect(actionQuit,SIGNAL(activated()),this,SLOT(close()));
  
  actionImportVRML = new QAction("VRML",tr(""),this);
  connect(actionImportVRML,SIGNAL(activated()),this,SLOT(slotImportVRML()));

  actionExportVRML = new QAction("VRML",tr(""),this);
  connect(actionExportVRML,SIGNAL(activated()),this,SLOT(slotExportVRML()));

  actionImportPGN = new QAction("PGN",tr(""),this);
  connect(actionImportPGN,SIGNAL(activated()),this,SLOT(slotImportPGN()));

  actionExportPGN = new QAction("PGN",tr(""),this);
  connect(actionExportPGN,SIGNAL(activated()),this,SLOT(slotExportPGN()));
  
  actionSimpleExtrusion = new QAction("Extrusion simple",Key_F9,this);
  connect(actionSimpleExtrusion,SIGNAL(activated()),this,SLOT(slotSimpleExtrusion()));

  actionProfileExtrusion = new QAction("Extrusion selon le profil",Key_F10,this);
  connect(actionProfileExtrusion,SIGNAL(activated()),this,SLOT(slotProfileExtrusion()));  
  
  actionTile = new QAction(tr("Tile"),tr("Ctrl+T"),this);
  actionTile->setIconSet(QPixmap::fromMimeSource("src/interface/icones/tile.png"));
  connect(actionTile,SIGNAL(activated()),this,SLOT(slotTile()));

  actionCascade = new QAction(tr("Cascade"),tr("Ctrl+D"),this);
  actionCascade->setIconSet(QPixmap::fromMimeSource("src/interface/icones/cascade.png"));
  connect(actionCascade,SIGNAL(activated()),this,SLOT(slotCascade()));
  
  changeWin = new QAction(tr("Changement de fentre"),Key_Tab,this);
  changeWin->setIconSet(QPixmap::fromMimeSource("src/interface/icones/change.png"));
  connect(changeWin,SIGNAL(activated()),this,SLOT(changeWindow()));
  connect(workspace,SIGNAL(windowActivated(QWidget*)),this,SLOT(updateStatusBar()));
  
  clearWindow = new QAction(tr("Effacer la fentre active"),Key_F5,this);
  clearWindow->setIconSet(QPixmap::fromMimeSource("src/interface/icones/clear.png"));
  connect(clearWindow,SIGNAL(activated()),this,SLOT(clearActiveWindow()));

  timerSimpleExtrusion = new QTimer( this, "RefreshSE" );
  connect(timerSimpleExtrusion, SIGNAL(timeout()), this, SLOT(slotRefreshSimpleExtrusion()));

  timerProfileExtrusion = new QTimer( this, "RefreshPE" );
  connect(timerProfileExtrusion, SIGNAL(timeout()), this, SLOT(slotRefreshProfileExtrusion()));

  actionChangeSample = new QAction(tr("Changement d'echantillonage"), tr(""), this);
  actionChangeSample->setIconSet(QPixmap::fromMimeSource("src/interface/icones/ok.png"));
  connect(actionChangeSample, SIGNAL(activated()), this, SLOT(slotChangeSample())) ;
}



void MainWindow::createMenus()
{


  /**************************************************Menu Fichier*******************************************************/
  menuFile = new QPopupMenu(this);
  
  actionNew->addTo( menuFile);
  actionOpen->addTo( menuFile);
  actionSave->addTo( menuFile);
  actionSaveAs->addTo( menuFile);
  menuFile->insertSeparator();
  
  //sous-menu importer
  subMenuImport = new QPopupMenu(this);
  actionImportVRML->addTo(subMenuImport);	
  actionImportPGN->addTo(subMenuImport);
  menuFile->insertItem("Importer",subMenuImport);
  
  //sous-menu exporter	
  subMenuExport = new QPopupMenu(this);
  actionExportVRML->addTo(subMenuExport);	
  actionExportPGN->addTo(subMenuExport);
  menuFile->insertItem("Exporter",subMenuExport);
  
  menuFile->insertSeparator();
  
  actionQuit->addTo(menuFile);

  menuBar()->insertItem(tr("Fichier"),menuFile);
 
 /**************************************************Menu Primitives*******************************************************/

  menuPrimitives = new QPopupMenu(this);
  
  actionSegment->addTo( menuPrimitives);
  actionCircle->addTo( menuPrimitives);
  actionBezier->addTo( menuPrimitives);
  actionBSpline->addTo( menuPrimitives);
    
  menuPrimitives->insertSeparator();
  
  menuBar()->insertItem(tr("Primitives"),menuPrimitives);
  
  /**************************************************Menu Extrusion*******************************************************/
  menuExtrusion = new QPopupMenu(this);
  
  actionSimpleExtrusion->addTo(menuExtrusion);
  actionProfileExtrusion->addTo(menuExtrusion);
  
  menuBar()->insertItem(tr("Extrusion"),menuExtrusion);

  /**************************************************Menu Affichage*******************************************************/
  menuDisplay = new QPopupMenu(this);
  
  clearWindow->addTo(menuDisplay);
  changeWin->addTo(menuDisplay);
  menuDisplay->insertSeparator();
  actionTile->addTo(menuDisplay);
  actionCascade->addTo(menuDisplay);
  
  menuBar()->insertItem(tr("Affichage"),menuDisplay);

  /**************************************************Menu Aide************************************************************/
  menuHelp = new QPopupMenu(this);
  
  menuHelp->insertItem("Aide",this,SLOT(slotHelp()));
  menuHelp->insertSeparator();
  menuHelp->insertItem("A propos du GCG",this,SLOT(slotAbout()));
  menuHelp->insertItem("A propos de Qt",qApp,SLOT(aboutQt()));
  menuBar()->insertItem(tr("Aide"),menuHelp);
}


void MainWindow::createToolsBar()
{
  barFile = new QToolBar( tr("Fichier"),this);
  actionNew->addTo( barFile );
  actionOpen->addTo( barFile );
  actionSave->addTo( barFile );

  barPrimitives = new QToolBar( tr("Primitives"),this);
  actionSegment->addTo(barPrimitives);
  actionCircle->addTo(barPrimitives);
  actionBezier->addTo(barPrimitives);
  actionBSpline->addTo(barPrimitives);  

  barDisplay = new QToolBar( tr("Affichage"),this);
  clearWindow->addTo(barDisplay);
  changeWin->addTo(barDisplay);
  actionTile->addTo(barDisplay);
  actionCascade->addTo(barDisplay);

  barTwist = new QToolBar( tr("Torsion"),this);
  QLabel *labelTwist;
  labelTwist = new QLabel("Torsion (degrs)", barTwist);
  spinboxTwist = new QSpinBox(barTwist, "torsion");
  spinboxTwist->setMaxValue( 360 * 5 );
  spinboxTwist->setMinValue( - 360 * 5 );

  barSample = new QToolBar( tr("Echantillonage"), this);
  QLabel *labelSample;
  labelSample = new QLabel("Echantillonage", barSample);
  sampleComboBox = new QComboBox(barSample, "sampleComboBox");
  sampleComboBox->insertItem("FAIBLE", 0);
  sampleComboBox->insertItem("NORMAL", 1);
  sampleComboBox->insertItem("FORT", 2);
  sampleComboBox->setCurrentItem(1);
  actionChangeSample->addTo(barSample);
}

void MainWindow::createStatusBar()
{
   infos = new QLabel(this);
   statusBar()->addWidget(infos);
   updateStatusBar();
}



void MainWindow::updateStatusBar()
{
   QString info = "fentre ";
   
   if (this->getWorkspace()->activeWindow())
     {
       QString windowName = this->getWorkspace()->activeWindow()->name();
       stringstream nameTypePrim;
       int typePrim = SEGMENT;
       
       if (windowName == "axe")
         typePrim = axis->getTypePrimitive();
       
       if (windowName == "section")
	 typePrim = section->getTypePrimitive();
       
       if (windowName == "profil")
	   typePrim = profile->getTypePrimitive();
       
       if (windowName != "visualisation 3D")
       { 
       switch (typePrim)
         {
	   case SEGMENT : nameTypePrim << "Primitive SEGMENT"; break; 
	   case CIRCLE : nameTypePrim << "Primitive CERCLE"; break;
	   case BEZIER : nameTypePrim << "Primitive BEZIER"; break;
	   case BSPLINE : nameTypePrim << "Primitive BSPLINE"; break;
	   default : break;
	 }
       
       info += windowName + " --> " + nameTypePrim.str();
       }
       else
         info = "fentre : visualisation 3D";
     }
   else
     info = "fentre : axe --> Primitive SEGMENT";
   
   infos->setText(info);
}



void MainWindow::slotNew()
{
  filename = "nouveau";
  spinboxTwist->setValue(0);
  sampleComboBox->setCurrentItem(1);
  stringstream str;
  str << "Gnrateur de Cylindres Gnraliss - " << filename;
  setCaption(str.str());
  profile->clear();
  section->clear();
  axis->clear();  
  opengl->clear();
  timerSimpleExtrusion->stop();
  timerProfileExtrusion->stop();
}



void MainWindow::slotOpen()
{

  QString name = QFileDialog::getOpenFileName (QString::null, "XML (*.xml)", this);
  if (!name.isEmpty() ) 
    {
      Modelml mml;
      profile->load(mml.load(name)[0]);
      section->load(mml.load(name)[1]);
      axis->load(mml.load(name)[2]);
      opengl->clear();
      spinboxTwist->setValue(0);
      sampleComboBox->setCurrentItem(1);
      filename = name;
      stringstream str;
      str << "Gnrateur de Cylindres Gnraliss - " << filename;
      setCaption(str.str());
      timerSimpleExtrusion->stop();
      timerProfileExtrusion->stop();
    }
}  


void MainWindow::slotHelp()
{
  QMessageBox::about(this,
		     tr("Aide"),
		     tr("Gnrateur de Cylindres Gnraliss\tv1.0\n" "voir README.txt et CylinderGenerator/doc/index.html\n"));
}



void MainWindow::slotAbout()
{
  QMessageBox::about(this,
		     tr("A propos"),
		     tr("Gnrateur de Cylindres Gnraliss\tv1.0\n"
			"\nAuteurs :\n"
			"\tGUITTENY Fabrice\n"
			"\tIDIART Baptiste\n"
			"\tLE GOFF Erwan")
		     );
}


void MainWindow::closeEvent( QCloseEvent *e )
{
  QMainWindow::closeEvent( e );
}



void MainWindow::slotSave()
{
  if (filename != "nouveau")
    {
      Modelml mml;
      mml.save(filename, profile -> getPrimitivesList(), section -> getPrimitivesList(), axis -> getPrimitivesList());
    }
  else
      slotSaveAs();
}


	
void MainWindow::slotSaveAs()
{
  QString name = QFileDialog::getSaveFileName (QString::null, "XML (*.xml)", this);
  if (!name.isEmpty()) 
    {
      Modelml mml;
      if (name.contains(".xml") == 0)
	name += ".xml";
      mml.save(name, profile->getPrimitivesList(), section->getPrimitivesList(), axis->getPrimitivesList());
      filename = name;
      stringstream str;
      str << "Gnrateur de Cylindres Gnraliss - " << filename;
      setCaption(str.str());
    }
}


void MainWindow::slotTile()
{
  this->getWorkspace()->tile();
}


void MainWindow::slotCascade()
{
  this->getWorkspace()->cascade();
}


void MainWindow::slotDrawSegment()
{
  profile->setTypePrimitive(SEGMENT);
  axis->setTypePrimitive(SEGMENT);
  section->setTypePrimitive(SEGMENT);
  profile->update();
  section->update();
  axis->update();
  updateStatusBar();
}
 
 
void MainWindow::slotDrawCircle()
{
  profile->setTypePrimitive(CIRCLE);
  axis->setTypePrimitive(CIRCLE);
  section->setTypePrimitive(CIRCLE);
  profile->update();
  section->update();
  axis->update();
  updateStatusBar();
}

 
void MainWindow::slotDrawBezier()
{
  profile->setTypePrimitive(BEZIER);
  axis->setTypePrimitive(BEZIER);
  section->setTypePrimitive(BEZIER);
  profile->update();
  section->update();
  axis->update();
  updateStatusBar();
}


void MainWindow::slotDrawBSpline()
{
  profile->setTypePrimitive(BSPLINE);
  axis->setTypePrimitive(BSPLINE);
  section->setTypePrimitive(BSPLINE);
  profile->update();
  section->update();
  axis->update();
  updateStatusBar();
} 


void MainWindow::slotImportVRML()
{
  timerSimpleExtrusion->stop();
  timerProfileExtrusion->stop();
  QString vrmlFile = QFileDialog::getOpenFileName(QString::null, "VRML (*.wrl)", this);
  if (!vrmlFile.isEmpty())
    {
      VrmlParser *vrml = new VrmlParser();
      vrml->importVrml(vrmlFile);
      opengl->clear();
      opengl->load(vrml->getNbPoints(), vrml->getNbPolys(), vrml->getPointsList(), vrml->getReferencesList());
    }
}


void MainWindow::slotExportVRML()
{
   QString name = QFileDialog::getSaveFileName (QString::null, "VRML (*.wrl)", this);
   if ( (!name.isEmpty())&&(!opengl->isEmpty()) )
       {
	 if (name.contains(".wrl") == 0)
	   name += ".wrl";
         VrmlParser *vrml = new VrmlParser();
	 int nbPoints;
	 int nbPolys;
	 vector<Point3D*> pointsList;
	 vector<int> referencesList;
	 opengl->exportOGL(nbPoints, nbPolys, pointsList, referencesList);
	 vrml->setParameters(nbPoints, nbPolys, pointsList, referencesList);
	 vrml->exportVrml(name);
       }
}


void MainWindow::slotImportPGN()
{
  timerSimpleExtrusion->stop();
  timerProfileExtrusion->stop();
  QString file_PGN = QFileDialog::getOpenFileName(QString::null, "PGN (*.pgn)", this);
  if (!file_PGN.isEmpty())
    {
      PgnParser *pgn = new PgnParser();
      pgn->importPGN(file_PGN);
      opengl->clear();
      opengl->load(pgn->getNbPoints(), pgn->getNbPolys(), pgn->getPointsList(), pgn->getReferencesList());
    }
}


void MainWindow::slotExportPGN()
{
   QString name = QFileDialog::getSaveFileName (QString::null, "PGN (*.pgn)", this);
     if ( (!name.isEmpty()) &&(!opengl->isEmpty()) )
       {
	 if (name.contains(".pgn") == 0)
	   name += ".pgn";
	 PgnParser *pgn = new PgnParser();
	 int nbPoints;
	 int nbPolys;
	 vector<Point3D*> pointsList;
	 vector<int> referencesList;
	 opengl->exportOGL(nbPoints, nbPolys, pointsList, referencesList);
	 pgn->setParameters(nbPoints, nbPolys, pointsList, referencesList);
	 pgn->exportPGN(name);
       }
}


void MainWindow::slotSimpleExtrusion()
{
  timerProfileExtrusion->stop();
  opengl->clear();
  if (axis->getPrimitivesList().size() == 0) {
    QMessageBox *mbox = new QMessageBox(QString("ERREUR"),QString("Pas de primitives dans la fentre Axe!"),
					QMessageBox::Critical,QMessageBox::Ok,QMessageBox::NoButton,
					QMessageBox::NoButton,this);
    mbox->show();
  }
  else
    if (section->getPrimitivesList().size() == 0){
      QMessageBox *mbox = new QMessageBox(QString("ERREUR"),QString("Pas de primitives dans la fentre Section !"),
					  QMessageBox::Critical,QMessageBox::Ok,QMessageBox::NoButton,
					  QMessageBox::NoButton,this);
      mbox->show();
    }
    else
      {
	int twistAngle = spinboxTwist->value();
	axis->changeCoordinateSystemWindow();
	section->changeCoordinateSystemWindow();
	Cylinder generalizedCylinder = Cylinder(axis->getPrimitivesList(), section->getPrimitivesList());
	vector<Point3D *> points;
	vector<int> facets;
	generalizedCylinder.setClosedSection(section->closedShape);
	generalizedCylinder.setTwist(twistAngle);
	generalizedCylinder.simpleExtrusion(points, facets);
	opengl->load(points.size(), facets.size() / 3, points, facets);
	axis->changeCoordinateSystemReverseWindow();
	section->changeCoordinateSystemReverseWindow();
	timerSimpleExtrusion->start(1000);
      }
}

void MainWindow::slotProfileExtrusion()
{
  timerSimpleExtrusion->stop();
  opengl->clear();
  if (axis->getPrimitivesList().size() == 0)
    {
      QMessageBox *mbox = new QMessageBox(QString("ERREUR"),QString("Pas de primitives dans la fentre Axe!"),
					  QMessageBox::Critical,QMessageBox::Ok,QMessageBox::NoButton,
					  QMessageBox::NoButton,this);
      mbox->show();
    }
  else
    if (section->getPrimitivesList().size() == 0)
      {
	QMessageBox *mbox = new QMessageBox(QString("ERREUR"),QString("Pas de primitives dans la fentre Section !"),
					    QMessageBox::Critical,QMessageBox::Ok,QMessageBox::NoButton,
					    QMessageBox::NoButton,this);
	mbox->show();
      }
    else
      if (profile->getPrimitivesList().size() == 0)
	{
	  QMessageBox *mbox = new QMessageBox(QString("ERREUR"), QString("Pas de primitives dans la fentre Profil !"),
					      QMessageBox::Critical,QMessageBox::Ok, QMessageBox::NoButton, 
					      QMessageBox::NoButton, this);
	  mbox->show();
	}
    else
      {
	int twistAngle = spinboxTwist->value();
	axis->changeCoordinateSystemWindow();
	section->changeCoordinateSystemWindow();
	profile->changeCoordinateSystemWindow();
	Cylinder generalizedCylinder = Cylinder(axis->getPrimitivesList(), section->getPrimitivesList(), profile->getPrimitivesList());
	vector<Point3D *> points;
	vector<int> facets;	
	generalizedCylinder.setClosedSection(section->closedShape);
	generalizedCylinder.setTwist(twistAngle);
	generalizedCylinder.profileExtrusion(points,facets);
	opengl->load(points.size(), facets.size() / 3, points, facets);
	axis->changeCoordinateSystemReverseWindow();
	section->changeCoordinateSystemReverseWindow();
	profile->changeCoordinateSystemReverseWindow();
	timerProfileExtrusion->start(1000);
      }
}


void MainWindow::changeWindow()
{

   QString name = this->getWorkspace()->activeWindow()->name();
   if (name == "axe")
      section->setFocus();
   if (name == "section")
      profile->setFocus();
   if (name == "profil")
      opengl->setFocus();
   if (name == "visualisation 3D")
      axis->setFocus();
   updateStatusBar();
}
  
void MainWindow::clearActiveWindow()
{
  timerSimpleExtrusion->stop();
  timerProfileExtrusion->stop();
  QString name = this->getWorkspace()->activeWindow()->name();
  if (name == "axe")
    axis->clear();
  if (name == "section")
    section->clear();
  if (name == "profil")
    profile->clear();
  if (name == "visualisation 3D")
    opengl->clear();
}


void MainWindow::resizeEvent(QResizeEvent *)
{
  workspace->tile();
}
void MainWindow::slotRefreshSimpleExtrusion()
{
  if((profile->isModified()) || (section->isModified()) || (axis->isModified()))
    {
      slotSimpleExtrusion();
      profile->setModified();
      section->setModified();
      axis->setModified();
    }
}

void MainWindow::slotRefreshProfileExtrusion()
{
  if((profile->isModified()) || (section->isModified()) || (axis->isModified()))
    {
      slotProfileExtrusion();
      profile->setModified();
      section->setModified();
      axis->setModified();
    }
}


void MainWindow::slotChangeSample()
{
  int newSample = sampleComboBox->currentItem();
  int sample = NORMAL_SAMPLE;

  switch (newSample)
    {
    case 0 : sample = LOW_SAMPLE; break;
    case 1 : sample = NORMAL_SAMPLE; break;
    case 2 : sample = ACCURATE_SAMPLE; break;
    default : sample = NORMAL_SAMPLE; break;
    }
      
  axis->setSample(sample);
  section->setSample(sample);
  profile->setSample(sample);  
}
