/** @file mainWindow.h */

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                       /
// Projet : Gnrateur de cylindres gnraliss                                          /
//                                                                                       /
// Nom du fichier : mainWindow.h                                                         /
//                                                                                       /
// Creation : 20/11/2005                                                                 /
//                                                                                       /
// Description : La classe MainWindow regroupe l'ensemble des                            / 
//              fonctionnalits ncssaires  la gnration de cylindres gnraliss.    /
//              Il s'agit de la fenetre principale contenant 3 fenetres 2D               /
//              (axe, profil et section) et d'une fenetre 3D o est gnre le cylindre. /
//                                                                                       /
//                                                                                       /
//                                                                                       /            
// Auteurs : - Guitteny Fabrice                                                          /
//           - Idiart Baptiste                                                           /
//           - Le Goff Erwan                                                             /
//                                                                                       /
//////////////////////////////////////////////////////////////////////////////////////////

#ifndef MAIN_WINDOW_H
#define MAIN_WINDOW_H

#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qstatusbar.h>
#include <qmenubar.h>
#include <qlabel.h>
#include <qpopupmenu.h>
#include <qaction.h>
#include <qworkspace.h>
#include <qmainwindow.h>
#include <qspinbox.h>
#include <qcombobox.h>

#include "window2D.h"
#include "../qgl/window3D.h"
#include "../qgl/vrmlparser.h"
#include "../qgl/pgnparser.h"
#include "../cylinder/cylinder.h"


/** La classe MainWindow regroupe l'ensemble des                     
 *  fonctionnalits ncssaires  la gnration de cylindres gnraliss.    
 *  Il s'agit de la fenetre principale contenant 3 fenetres 2D               
 *  (axe, profil et section) et d'une fenetre 3D o est gnre le cylindre.
 *
 * @author Guitteny Fabrice
 * @author Idiart Baptiste
 * @author Le Goff Erwan
 */


///////////////////////////////////////////////////////////////////////////////////////////
//  Classe MainWindow                                                                    //
///////////////////////////////////////////////////////////////////////////////////////////

class MainWindow : public QMainWindow
{
  Q_OBJECT
public :
    
  MainWindow(QWidget *parent,const char *nom = 0);
  
  // destructeur
  ~MainWindow();

  /** @return pointeur sur l'espace de travail compos des 4 fentres. */
  QWorkspace * getWorkspace() {return workspace;}
  
  /** @return pointeur sur la fenetre 2D repsentant le profil. */
  Window2D * getProfileWindow() {return profile;}

  /** @return pointeur sur la fenetre 2D repsentant la section. */
  Window2D * getSectionWindown() {return section;}

 /** @return pointeur sur la fenetre 2D repsentant l'axe. */
  Window2D * getAxisWindow() {return axis;}

  /** @return pointeur sur la fenetre 3D. */
  Window3D * getOpenGLWondow() {return opengl;}
  

private slots:

  void slotAbout();
  void slotHelp(); 
  void slotNew();
  void slotOpen();
  void slotSave();
  void slotSaveAs();
  void slotDrawCircle();
  void slotDrawSegment();
  void slotDrawBezier();
  void slotDrawBSpline();
  void slotTile();
  void slotCascade();
  void slotImportVRML();
  void slotExportVRML();
  void slotImportPGN()	;
  void slotExportPGN();
  void slotSimpleExtrusion();
  void slotProfileExtrusion();
  void slotRefreshSimpleExtrusion();
  void slotRefreshProfileExtrusion();
  void slotChangeSample();
  
  void updateStatusBar();
  
  void closeEvent( QCloseEvent *e );
  void resizeEvent(QResizeEvent *);
  
  void changeWindow();
  void clearActiveWindow();


private:
  
  void createActions();
  void createMenus();
  void createToolsBar();
  void createStatusBar();
  
  QPopupMenu *menuFile;
  QAction *actionOpen;
  QAction *actionNew;
  QAction *actionSave;
  QAction *actionSaveAs;
  
  QPopupMenu *subMenuImport;
  QAction *actionImportVRML;
  QAction *actionImportPGN;
  
  QPopupMenu *subMenuExport;
  QAction *actionExportVRML;
  QAction *actionExportPGN;
  QAction *actionQuit;
  
  QPopupMenu *menuPrimitives;
  QAction *actionSegment;
  QAction *actionCircle;
  QAction *actionBezier;
  QAction *actionBSpline;
  
  QPopupMenu *menuExtrusion;
  QAction *actionSimpleExtrusion;
  QAction *actionProfileExtrusion;
  
  QPopupMenu *menuDisplay;
  QAction *actionTile;
  QAction *actionCascade;
  
  QPopupMenu *menuHelp;
  
  QToolBar *barFile;
  QToolBar *barPrimitives;
  QToolBar *barDisplay;
  
  QToolBar *barTwist;
  QSpinBox *spinboxTwist;

  QToolBar *barSample;
  QComboBox *sampleComboBox;

  QAction *actionChangeSample;

  QWorkspace *workspace;

  Window2D *profile;
  Window2D *section;
  Window2D *axis;
  Window3D *opengl;
  
  QAction *changeWin;
  QAction *clearWindow;
  QLabel* infos;

  /** nom du fichier en cours d'utilisation */
  QString filename;
  
  /** les changements dans les fentre de dessin sont rpercuts aprs
   *  un certain temps dfini par le timer pour l'extrusion simple
   */
  QTimer * timerSimpleExtrusion;

  /** idem pour l'extrusion profil */
  QTimer * timerProfileExtrusion;

};

#endif

