/** @file window2D.h */

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                       /
// Projet : Gnrateur de cylindres gnraliss                                          /
//                                                                                       /
// Nom du fichier : window2D.h                                                          /
//                                                                                       /
// Creation : 20/11/2005                                                                 /
//                                                                                       /
// Description : La classe window2D regroupe l'ensemble des fonctionnalits ncssaires /
//              au dessin de primitives.                                                 /
//              Les dessins s'effectuent sur des objets Qpainter                         /
//                                                                                       /
//                                                                                       /            
// Auteurs : - Guitteny Fabrice                                                          /
//           - Idiart Baptiste                                                           /
//           - Le Goff Erwan                                                             /
//                                                                                       /
//////////////////////////////////////////////////////////////////////////////////////////


#ifndef WINDOW2D_H
#define WINDOW2D_H

#include <qwidget.h>
#include <qpainter.h>
#include <qapplication.h>
#include <qmessagebox.h>

#include <stdlib.h>
#include <vector>
#include "math.h"

#include "../primitives/point.h"
#include "../primitives/segment.h"
#include "../primitives/circle.h"
#include "../primitives/bezier.h"
#include "../primitives/bspline.h"

#include "utilities.h"

#include"../qxml/modelml.h"

// nombre maximum de points dans la fentre

const int MAXPOINTS = 256;

// rsolution de la fentre

const int WIDTH = 640;
const int HEIGHT = 640;

/**
 * La classe Window2D regroupe l'ensemble des fonctionnalits ncssaires 
 * au dessin de primitives.                                                 
 * Les dessins s'effectuent sur des objets Qpainter  
 *
 * @author Guitteny Fabrice
 * @author Idiart Baptiste
 * @author Le Goff Erwan
 */

///////////////////////////////////////////////////////////////////////////////////////////
//  Classe Window2D                                                                      //
///////////////////////////////////////////////////////////////////////////////////////////

class Window2D : public QWidget
{

 private:

  /**liste des points dans la fentre */
  vector<QPoint> pointsList;  
  
  /** type de la primitive dessine */
  int typePrimitive;    
  
  /** nombre de points dfinis par l'utilisateur */
  int nbPoints;    
  
  /** vrai si la souris est presse */
  bool down;          
  
  /** liste des primitives de la fentre */
  vector<Primitive *> primitivesList; 
  
  /** nombre de primitives */
  int nbPrimitives;        
  
  /** vrai si l'utilisateur bouge un point */
  bool move;                 
  
  /** indice du point slectionn par l'utilisateur */
  int selectedPoint;     
  
  /** indice du dernier point cliqu par l'utilisateur */ 
  int indexCurrentPoint;   
  
  /** dernier point cliqu par l'utilisateur */
  QPoint currentPoint;  
  
  /** liste des derniers points slectionns pour dessiner une primitive */
  vector<Point3D *> lastPointsList; 
  
  /**  coordonnes de l'origine du repre (centre de la fentre) */
  Point3D origin;         
  
  /** gre le changement bspline - bzier */
  bool toBezier;

  /** pour tester s'il faut rafraichir la fenetre OpenGL */
  bool modification; 

  /** nombre d'chantillons des primitives */
  int sample;


 public:

  /** pour savoir si la forme dessine est ferme */
  bool closedShape;

  /** constructeur par dfaut */
  Window2D( QWidget *parent=0, const char *name = 0);

  ~Window2D();

  /** accesseur aux diffrents types de primitives utilises
   * @return vecteur de pointeur sur des Primitives 
   * @see primitive.h
   */
  inline vector<Primitive *> getPrimitivesList() {return primitivesList;}

  /** accesseur au type de la primitive courante
   * @return entier correspondant  une primitive
   * @see primitive.h
   */
  inline int getTypePrimitive() {return typePrimitive;}

  /** @return entier correspondant au nombre de primitives
   *  @see primitive.h
   */
  inline int getNbPrimitives() {return nbPrimitives;}

  /** @return entier correspondant au nombre de points */
  inline int getNbPoints() {return nbPoints;}

   /** accesseur : valeur de l'chantillonage
   * @return nombre d'chantillons
   */
  inline int getSample() {return sample;}

  /** modificateur : valeur de l'chantillonage
   * @param s nouvelle valeur de l'chantillonage
   */
  inline void setSample(int s)
    {
      sample = s;
      for (int i = 0; i < nbPrimitives; i++)
	primitivesList[i]->setNbSamples(sample);
      update();
    }
 
  /** @return boolen indiquant s'il faut rafraichir la fenetre OpenGL */
  inline bool isModified() {return modification;}
 
  /** change le type courant de primitive
   * @param typePrim entier correspondant  un type de primitives 
   */
  inline void setTypePrimitive( const int typePrim ) {typePrimitive = typePrim;}
  
  /** charge des primitives dans la fentre.
   * @param prim vecteur de primitives.
   */
  void load(vector<Primitive *> &prim);
  
  /** rinitialisation de la fentre */
  void clear();
  
  /** changement de repre par rapport au centre de la fentre */
  void changeCoordinateSystemWindow();

  /** changement de repre inverse par rapport au centre de la fentre */
  void changeCoordinateSystemReverseWindow();
  
  inline void setModified(bool modified = false){modification = modified;}

 protected:

  /** efface les vecteurs de la fentre */ 
  void clearVectors();

  /** dessin de la fentre */
  void paintEvent( QPaintEvent * );
  void mousePressEvent( QMouseEvent *);
  void mouseReleaseEvent( QMouseEvent *);
  void mouseMoveEvent( QMouseEvent *);
  void closeEvent(QCloseEvent * );
  
};

#endif

