#include "bezier.h"

// constructeurs

Bezier::Bezier()
{
  type = BEZIER;
  nbSamples = NORMAL_SAMPLE;
}

Bezier::Bezier(const vector<Point3D *>& ptsList)
{
  for (unsigned int i = 0; i < ptsList.size(); i++)
    pointsList.push_back(ptsList[i]);
  type = BEZIER;
  nbSamples = NORMAL_SAMPLE;
}

Bezier::Bezier(const Bezier& bez) : Primitive(bez)
{
  vector<Point3D *> vect = bez.pointsList;
  for (unsigned int i = 0; i < vect.size(); i++)
    pointsList.push_back(vect[i]);
  type = BEZIER;
  nbSamples = NORMAL_SAMPLE;
}


// accesseurs
vector<Point3D *>
Bezier::getCtrlPoints() const
{
  return pointsList;
}

// modificateurs
void
Bezier::addPoint(Point3D * point)
{
  pointsList.push_back(point);  
}

void
Bezier::addPointPosI(Point3D * point, const unsigned int pos)
{
  if (pos < pointsList.size())    // on peut insrer
    {
      vector<Point3D *> temp;
      for (unsigned int i = 0; i < pos; i++)
	temp.push_back(pointsList[i]);
      temp.push_back(point);
      for (unsigned int i = pos + 1; i <= pointsList.size(); i++)
	temp.push_back(pointsList[i-1]);
      pointsList = temp;
    }
}



// mthode d'affichage d'une courbe de Bzier

void
Bezier::display(QPainter& paint) {
 
  Point3D p0 = *pointsList[0];
  Point3D p1 = *pointsList[1];
  Point3D p2 = *pointsList[2];
  Point3D p3 = *pointsList[3];
  
  int temp = 0;
  samplesPointsList.clear();

  // value la position au temps t
  for (float t = 0.0f; t <= 1.0f; t += 0.001)
    {
      // bases de Bernstein
      float B0 = (1-t)*(1-t)*(1-t);
      float B1 = 3*t*(1-t)*(1-t);
      float B2 = 3*t*t*(1-t);
      float B3 = t*t*t;
      
      // pondration
      Point3D pos = p0*B0 + p1*B1 + p2*B2 + p3*B3;

      if (temp >= nbSamples)
	{
	  samplesPointsList.push_back(new Point3D(pos));
	  temp = 0;
	}
      temp++; 
      paint.drawPoint((int)pos.getCoordinate(0),(int)pos.getCoordinate(1));
    }
}



// changements des points de contle de la courbe de Bzier
void
Bezier::setPrimitive(QPoint & refPoint, QPoint & modifPoint)
{
  Point3D * p = qPointToPoint3D(refPoint);
  if (*p == *pointsList[0])
    pointsList[0] = qPointToPoint3D(modifPoint);
  if (*p == *pointsList[1])
    pointsList[1] = qPointToPoint3D(modifPoint);
  if (*p == *pointsList[2])
    pointsList[2] = qPointToPoint3D(modifPoint);
  if (*p == *pointsList[3])
    pointsList[3] = qPointToPoint3D(modifPoint);
}



// ecrit une primtive dans le fichier xml
void Bezier::toXML(QDomElement &elem)
{
  string nomAttr;
  for(int i = 0 ;i<nbPoints() ; i++)
    {
      char snumpt[256]="";
      sprintf(snumpt,"%d",i);
      string temp = (string)snumpt;
      nomAttr = "P"+temp+"x";
      elem.setAttribute( nomAttr, (*pointsList[i])[0]);
      char snumpt2[256]="";
      sprintf(snumpt2,"%d",i);
      string temp2 = (string)snumpt2;
      nomAttr = "P"+temp2+"y";
      elem.setAttribute( nomAttr, (*pointsList[i])[1]);
    }  
}



