/** @file bezier.h */

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                       /
// Projet : Gnrateur de cylindres gnraliss                                          /
//                                                                                       /
// Nom du fichier : bezier.h                                                             /
//                                                                                       /
// Creation : 20/11/2005                                                                 /
//                                                                                       /
// Description : La classe Bezier dessine des bziers cubiques.                          / 
//                                                                                       /
// Auteurs : - Guitteny Fabrice                                                          /
//           - Idiart Baptiste                                                           /
//           - Le Goff Erwan                                                             /
//                                                                                       /
//////////////////////////////////////////////////////////////////////////////////////////

#ifndef BEZIER_H
#define BEZIER_H

#include<iostream>
#include <qpainter.h>
#include <vector>
#include "primitive.h"

using namespace std;

/** La classe Bezier dessine des bziers cubiques.                       
 *
 * @author Guitteny Fabrice
 * @author Idiart Baptiste
 * @author Le Goff Erwan
 */

///////////////////////////////////////////////////////////////////////////////////////////
//  Classe Bezier                                                                        //
///////////////////////////////////////////////////////////////////////////////////////////

class Bezier : public Primitive
{  
  public :
  
    /** constructeur par dfaut */
    Bezier();

  /** constructeur
   * @param pointsList liste de points de contrle
   */
  Bezier(const vector<Point3D *>& pointsList);

  /** constructeur par copie
   * @param bez bezier
   */
  Bezier(const Bezier& bez);
  
  // destructeur
  ~Bezier() {};

  /** mthode d'affichage d'une courbe de Bzier
   * @param paint affichage dans un QPainter
   */
  virtual void display(QPainter& paint); 

  /** changement des points de contrle de la courbe de Bzier
   * @param refPoint point  modifier
   * @param modifPoint nouvelle valeur du point
   */
  virtual void setPrimitive(QPoint & refPoint, QPoint & modifPoint);
  
  /** ecrit la primitive dans un fichier xml
   * @param elem balise courante dans le fichier XML
   */
  virtual  void toXML(QDomElement &elem);  
  
  /** accesseur
   * @return liste de points de contrle
   */
  vector<Point3D *> getCtrlPoints() const;
  
  /** ajoute un point de contrle  la liste de points de contrle
   * @param point point  ajouter
   */
  void addPoint(Point3D * point);

  /** ajoute un point de contrle  une position dfini de la liste de points de contrle
   * @param point point  ajouter
   * @param pos position dans la liste
   */
  void addPointPosI(Point3D *point, const unsigned int pos);

};

#endif

