#include "bspline.h"

// constructeurs
Bspline::Bspline()
{
  type = BSPLINE;
  nbSamples = NORMAL_SAMPLE;
}

Bspline::Bspline(const vector<Point3D *>& lastPointsList)
{
  for (unsigned int i = 0; i < lastPointsList.size(); i++)
    pointsList.push_back(lastPointsList[i]);
  type = BSPLINE;
  nbSamples = NORMAL_SAMPLE;
}

Bspline::Bspline(const Bspline& spl) : Primitive(spl)
{
  vector<Point3D *> vect = spl.pointsList;
  for (unsigned int i = 0; i < vect.size(); i++)
    pointsList.push_back(vect[i]);
  type = BSPLINE;
  nbSamples = NORMAL_SAMPLE;
}



// accesseurs

vector<Point3D *>
Bspline::getCtrlPointsList() const
{
  return pointsList;
}

// modificateurs
void
Bspline::addPoint(Point3D * point)
{
  pointsList.push_back(point);  
}

void
Bspline::addPointPosI(Point3D * point, const unsigned int pos)
{
  if (pos < pointsList.size())    // on peut insrer
    {
      vector<Point3D *> temp;
      for (unsigned int i = 0; i < pos; i++)
	temp.push_back(pointsList[i]);
      temp.push_back(point);
      for (unsigned int i = pos + 1; i <= pointsList.size(); i++)
	temp.push_back(pointsList[i-1]);
      pointsList = temp;
    }
}



// fonctions de cration des bsplines

double
Bspline::blend(int k, int t, int * u, double v)
{
  double valeur;
  
  if (t == 1)
    {
      if ((u[k] <= v) && (v < u[k+1]))
	valeur = 1;
      else
	valeur= 0;
    }
  else
    {
      if ((u[k+t-1] == u[k]) && (u[k+t] == u[k+1]))
	valeur = 0;
      else
	if (u[k+t-1] == u[k]) 
	  valeur = (u[k+t] - v) / (u[k+t] - u[k+1]) * blend(k+1,t-1,u,v);
	else
	  if (u[k+t] == u[k+1])
	    valeur = (v - u[k]) / (u[k+t-1] - u[k]) * blend(k,t-1,u,v);
	  else
	    valeur = (v - u[k]) / (u[k+t-1] - u[k]) * blend(k,t-1,u,v) + 
	      (u[k+t] - v) / (u[k+t] - u[k+1]) * blend(k+1,t-1,u,v);
    }
  return(valeur);
}



Point3D
Bspline::calculatePoint(int * u, int n, int t, double v)
{
  double xx = 0;
  double yy = 0;
  double zz = 0;
  
  for (int k = 0; k <= n; k++)
    {
      double b = blend(k, t, u, v);
      xx += pointsList[k]->getCoordinate(0) * b;
      yy += pointsList[k]->getCoordinate(1) * b;
      zz += pointsList[k]->getCoordinate(2) * b;
    }

  return Point3D(xx,yy,zz);
}



void
Bspline::calculateNodes(int * u, int n, int t)
{
  int j;

  for (j=0;j<=n+t;j++) {
    if (j < t)
      u[j] = 0;
    else if (j <= n)
      u[j] = j - t + 1;
    else if (j > n)
      u[j] = n - t + 2;	
  }
}


// mthode d'affichage d'une bspline
void
Bspline::display(QPainter& paint)
{
  int size = pointsList.size();
  int degree = 4;
  int nodes[size+degree+1];
  calculateNodes(nodes, size-1, degree);
 
  double interval = 0;
  int precision = 250 * pointsList.size();
  double increment = (double) ( (double) (size - 1 - degree + 2) / (double) (precision - 1) );

  int temp = 0;

  samplesPointsList.clear();

  for (int i = 0; i < precision; i++)
    {
      Point3D res = calculatePoint(nodes, size - 1, degree, interval);
      interval += increment;
      
      if (temp >= nbSamples)
	{
	  samplesPointsList.push_back(new Point3D(res));
	  temp = 0;
	}
      temp++;
      paint.drawPoint( (int)(res.getCoordinate(0)), (int)(res.getCoordinate(1)) );
    }
  samplesPointsList.push_back(new Point3D(*pointsList[size-1]));
}

// changements des points de contle de la bspline
void
Bspline::setPrimitive(QPoint & pointRef, QPoint & pointModif)
{
  Point3D * p = qPointToPoint3D(pointRef);  
  for (unsigned int i = 0; i < pointsList.size(); i++)
    if (*p == *pointsList[i])
      pointsList[i] = qPointToPoint3D(pointModif);
}



// ecrit une primtive dans le fichier xml
void
Bspline::toXML(QDomElement &elem)
{
  string attributeName;
  for(int i = 0 ;i<nbPoints() ; i++)
    {
      char snumpt[256]="";
      sprintf(snumpt,"%d",i);
      string temp = (string)snumpt;
      attributeName = "P"+temp+"x";
      elem.setAttribute( attributeName, (*pointsList[i])[0]);
      char snumpt2[256]="";
      sprintf(snumpt2,"%d",i);
      string temp2 = (string)snumpt2;
      attributeName = "P"+temp2+"y";
      elem.setAttribute( attributeName, (*pointsList[i])[1]);
    }
}

