/** @file bspline.h */

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                       /
// Projet : Gnrateur de cylindres gnraliss                                          /
//                                                                                       /
// Nom du fichier : bspline.h                                                            /
//                                                                                       /
// Creation : 20/11/2005                                                                 /
//                                                                                       /
// Description : La classe Bspline permet de spcifier et de dessiner des bsplines.      /
//                                                                                       /            
// Auteurs : - Guitteny Fabrice                                                          /
//           - Idiart Baptiste                                                           /
//           - Le Goff Erwan                                                             /
//                                                                                       /
//////////////////////////////////////////////////////////////////////////////////////////

#ifndef BSPLINE_H
#define BSPLINE_H

#include<iostream>
#include <qpainter.h>
#include <vector>
#include "primitive.h"

using namespace std;

/**
 * La classe Bspline permet de spcifier et de dessiner des bsplines.
 *
 *  @author Guitteny Fabrice
 *  @author Idiart Baptiste
 *  @author Le Goff Erwan
 */

///////////////////////////////////////////////////////////////////////////////////////////
//  Classe Bspline                                                                       //
///////////////////////////////////////////////////////////////////////////////////////////

class Bspline : public Primitive
{  
  
  public :
    
    /** constructeur par dfaut */
    Bspline();
  
  /** constructeur
   * @param lPoints liste de points de contrle
   */
  Bspline(const vector<Point3D *>& lPoints);
  
  /** constructeur par copie
   * @param spl bspline
   */
  Bspline(const Bspline& spl);
  
  // destructeur
  ~Bspline() {};

  /** mthode d'affichage d'une Bspline
   * @param paint affichage dans un QPainter
   */
  virtual void display(QPainter& paint);  
  
  /** changement des points de contrle de la Bspline
   * @param refPoint point  modifier
   * @param modifPoint nouvelle valeur du point
   */
  virtual void setPrimitive(QPoint & refPoint, QPoint & modifPoint);  

  /** ecrit la primitive dans un fichier xml
   * @param elem balise courante dans le fichier XML
   */
  virtual  void toXML(QDomElement &elem);

  /** accesseur
   * @return liste de points de contrle
   */
  vector<Point3D *> getCtrlPointsList() const;

  /** ajoute un point de contrle  la liste de points de contrle
   * @param point point  ajouter
   */
  void addPoint(Point3D * point);

  /** ajoute un point de contrle  une position dfini de la liste de points de contrle
   * @param point point  ajouter
   * @param pos position dans la liste
   */
  void addPointPosI(Point3D *point, const unsigned int pos);
  
  private :
    
    // fonctions de cration des bsplines
    double blend(int k, int t, int * u, double v);
  Point3D calculatePoint(int * u, int n, int t, double v);
  void calculateNodes(int * u, int n, int t);
  
};

#endif

