#include "circle.h"

// structure de tri pour la liste d'chantillons
typedef struct order
{
  bool operator()(Point3D * pA, Point3D * pB) const
  {
    Point3D p1 = *pA;
    Point3D p2 = *pB;
    
    if (p1[1] >= 0)
      if (p2[1] >= 0)
	return (p1[0] <= p2[0]);
      else
	return false;
    else
      if (p2[1] >= 0)
	return true;
      else
	return (p1[0] > p2[0]);  
  }  
} order;

// constructeurs
Circle::Circle()
{
  type = CIRCLE;
  nbSamples = NORMAL_SAMPLE / 3;
}

Circle::Circle(Point3D * center, const double& r) : radius(r)
{
  pointsList.push_back(center);
  Point3D * radiusPts = new Point3D( center->getCoordinate(0) + radius,
				     center->getCoordinate(1),
				     center->getCoordinate(2) );
  pointsList.push_back(radiusPts);
  type = CIRCLE;
  nbSamples = NORMAL_SAMPLE / 3;
}

Circle::Circle(Point3D * center, Point3D * radiusPts)
{
  pointsList.push_back(center);
  pointsList.push_back(radiusPts);
  calculateRadius();
  type = CIRCLE;
  nbSamples = NORMAL_SAMPLE / 3;
}

Circle::Circle(const Circle& c) : Primitive(c)
{
  Point3D * center = c.pointsList[0];
  Circle(center,c.radius);
}


// accesseurs

Point3D *
Circle::getCenter() 
{
  return pointsList[0];
}

Point3D *
Circle::getRadiusPoint() 
{
  return pointsList[1];
}

double
Circle::getRadius() const
{
  return radius;
}

double&
Circle::getRadius()
{
  return radius;
}

// modificateurs
void
Circle::setCenter(Point3D * c)
{
  pointsList[0] = c;
}

void
Circle::setRadiusPoint(Point3D * pr)
{
  pointsList[1] = pr;
}

// calcul du rayon  partir du centre et d'un autre point
void
Circle::calculateRadius()
{
  Point3D * center = pointsList[0];
  Point3D * radiusPts = pointsList[1];
  radius = sqrt((double)((center->getCoordinate(0) - radiusPts->getCoordinate(0)) * (center->getCoordinate(0) - radiusPts->getCoordinate(0)) 
			 + (center->getCoordinate(1) - radiusPts->getCoordinate(1)) * (center->getCoordinate(1) - radiusPts->getCoordinate(1))
			 + (center->getCoordinate(2) - radiusPts->getCoordinate(2)) * (center->getCoordinate(2) - radiusPts->getCoordinate(2)) ) );
}


// mthode d'affichage d'un cercle (Bresenham)
void
Circle::display(QPainter & paint)
{
  int x,y,d ;
  x = 0;
  y = (int)radius ;
  d = 1 - (int)radius ;

  int xcenter = (int)pointsList[0]->getCoordinate(0);
  int ycenter = (int)pointsList[0]->getCoordinate(1);
  
  int temp = 0;
    
  samplesPointsList.clear();
 
  paint.drawPoint(x + xcenter, y + ycenter);
  paint.drawPoint(x + xcenter, -y + ycenter);
  paint.drawPoint(y + xcenter, x + ycenter);
  paint.drawPoint(-y + xcenter, x + ycenter);
  
  while ( y > x )
    {
      if ( d < 0 )
	d += 2 * x + 3 ;
      else
	{
	  d += 2 * (x - y) + 5 ;
	  y-- ;
	}
      x++ ;
      
      if (temp >= nbSamples)
	{
	  samplesPointsList.push_back(new Point3D(x, y, 0));
	  samplesPointsList.push_back(new Point3D(-x, y, 0));
	  samplesPointsList.push_back(new Point3D(x, -y, 0));
	  samplesPointsList.push_back(new Point3D(-x, -y, 0));
	  samplesPointsList.push_back(new Point3D(y, x, 0));
	  samplesPointsList.push_back(new Point3D(-y, x, 0));
	  samplesPointsList.push_back(new Point3D(y, -x, 0));
	  samplesPointsList.push_back(new Point3D(-y, -x, 0));
	  
	  temp = 0;
	}
      temp++;
      
      paint.drawPoint(x + xcenter, y + ycenter);
      paint.drawPoint(-x + xcenter, y + ycenter);
      paint.drawPoint(x + xcenter, -y + ycenter);
      paint.drawPoint(-x + xcenter, -y + ycenter);
      paint.drawPoint(y + xcenter, x + ycenter);
      paint.drawPoint(-y + xcenter, x + ycenter);
      paint.drawPoint(y + xcenter, -x + ycenter);
      paint.drawPoint(-y + xcenter, -x + ycenter);
      
    }

  // les points d'chantillons n'tant pas gnrs  la suite
  // (symtrie dans l'algorithme de Bresenham), il faut trier
  // la liste afin qu'ils soient dans l'ordre (pour la future 
  // gnration des facettes lors des extrusions).

  sortSamplesPointsList();

  // le tri s'effectuant pour un cercle centr  l'origine,
  // il faut donc changer de repre ie ajouter les coordonnes
  // du centre  chaque point d'chantillon
  
  for (unsigned int i = 0; i < samplesPointsList.size(); i++)
    {
      (*samplesPointsList[i])[0] += xcenter;
      (*samplesPointsList[i])[1] += ycenter;
    }

  // on ajoute le premier poit  la fin afin que la forme soit ferme
  samplesPointsList.push_back(samplesPointsList[0]);
}



// changement des points dfinissant un cercle
void
Circle::setPrimitive(QPoint & refPoint, QPoint & modifPoint)
{
  Point3D * p = qPointToPoint3D(refPoint);
  if (*p == *getCenter())
    setCenter(qPointToPoint3D(modifPoint));
  if (*p == *getRadiusPoint())
    setRadiusPoint(qPointToPoint3D(modifPoint));
  calculateRadius();
}



// ecrit une primitive dans le fichier xml
void Circle::toXML(QDomElement &elem)
{
  elem.setAttribute( "Ax", ( *getCenter())[0] );
  elem.setAttribute( "Ay", ( *getCenter())[1] );
  elem.setAttribute( "Bx", ( *getRadiusPoint())[0] );
  elem.setAttribute( "By", ( *getRadiusPoint())[1] );
}



// tri des points d'chantillons
void
Circle::sortSamplesPointsList()
{
  sort(samplesPointsList.begin(), samplesPointsList.end(), order());
}

