/** @file circle.h */

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                       /
// Projet : Gnrateur de cylindres gnraliss                                          /
//                                                                                       /
// Nom du fichier : circle.h                                                             /
//                                                                                       /
// Creation : 20/11/2005                                                                 /
//                                                                                       /
// Description : La classe Circle permet de spcifier et de dessiner des cercles.        / 
//                                                                                       /
// Auteurs : - Guitteny Fabrice                                                          /
//           - Idiart Baptiste                                                           /
//           - Le Goff Erwan                                                             /
//                                                                                       /
//////////////////////////////////////////////////////////////////////////////////////////

#ifndef CIRCLE_H
#define CIRCLE_H

#include<iostream>
#include <qpainter.h>
#include "primitive.h"
using namespace std;

/** La classe Circle permet de spcifier et de dessiner des cercles.                       
 *
 * @author Guitteny Fabrice
 * @author Idiart Baptiste
 * @author Le Goff Erwan
 */

///////////////////////////////////////////////////////////////////////////////////////////
//  Classe Circle                                                                        //
///////////////////////////////////////////////////////////////////////////////////////////

class Circle : public Primitive {

  protected :

    /** rayon du cercle */
    double radius;

  public :
  
    /** constructeur par dfaut */
    Circle();

  /** constructeur : centre et rayon
   * @param c centre du cercle
   * @param r rayon du cercle
   */	
  Circle(Point3D * c, const double& r);

  /** constructeur : centre et point du cercle
   * @param center centre du cercle
   * @param p_radius point sur le cercle
   */	
  Circle(Point3D * center, Point3D * p_radius);

  /** constructeur par copie
   * @param c cercle
   */
  Circle(const Circle& c);

  // destructeur
  ~Circle() {};

  /** mthode d'affichage d'un cercle (Bresenham)
   * @param paint affichage dans un QPainter
   */
  virtual void display (QPainter& paint);

  /** changement des points dfinissant un cercle
   * @param refPoint point  modifier
   * @param modifPoint nouvelle valeur du point
   */
  virtual void setPrimitive(QPoint & refPoint, QPoint & modifPoint);
  
  /** ecrit la primitive dans un fichier xml
   * @param elem balise courante dans le fichier XML
   */
  virtual  void toXML(QDomElement &elem);
  
  /** accesseur : centre
   * @return centre du cercle
   */
  Point3D * getCenter();

  /** accesseur : point sur le cercle
   * @return point sur le cercle
   */
  Point3D * getRadiusPoint();

  /** accesseur : rayon du cercle
   * @return rayon du cercle
   */
  double getRadius() const;

  /** modificateur : rayon du cercle
   * @return rayon du cercle
   */
  double& getRadius();
  
  /** modificateur : centre du cercle
   * @param c nouveau centre du cercle
   */
  void setCenter(Point3D * c);

  /** modificateur : point sur le cercle
   * @param pr nouveau point sur le cercle
   */
  void setRadiusPoint(Point3D * pr);

  /** calcul du rayon  partir du centre et d'un point sur le cercle */
  void calculateRadius();

  /** tri de la liste d'chantillons */
  void sortSamplesPointsList();

};

#endif

