#include "point.h"

// Constructeurs

Point3D::Point3D(const Point3D &point)
{
  x = point[0]; y = point[1]; z = point[2];
}

Point3D::Point3D(const double xx, const double yy, const double zz)
{
  x = xx; y = yy; z = zz;
}



// Accesseurs

double
Point3D::operator[](int i) const
{
  switch (i)
    {
    case 0 : return x;
    case 1 : return y;
    case 2 : return z;
    default : return x;
    }
}

double&
Point3D::operator[](int i)
{
  switch (i)
    {
    case 0 : return x;
    case 1 : return y;
    case 2 : return z;
    default : return x;
    }
}

double&
Point3D::getCoordinate(int i)
{
  switch (i)
    {
    case 0 : return x;
    case 1 : return y;
    case 2 : return z;
    default : return x;
    }
}



// calcul de la distance
double
Point3D::distance (Point3D &point)
{
  double xtemp = x - point.getCoordinate(0);
  double ytemp = y - point.getCoordinate(1);
  double ztemp = z - point.getCoordinate(2);
  
  return sqrt( (xtemp * xtemp)
	       + (ytemp * ytemp)
	       + (ztemp * ztemp) );
}



// Operateurs internes

Point3D&
Point3D::operator*=(const double scalar)
{
  x *= scalar; y *= scalar; z *= scalar;
  return *this;
}

Point3D&
Point3D::operator+=(const Point3D &point)
{
  x += point[0]; y += point[1]; z += point[2];
  return *this;
}

Point3D&
Point3D::operator-=(const Point3D &point)
{
  x -= point[0]; y -= point[1]; z -= point[2];
  return *this;
}



// Operateurs externes

std::ostream&
operator<<(std::ostream &os, const Point3D &point)
{
  os << "(" << point[0] << "," <<  point[1] <<  "," <<  point[2] << ")";
  return os;
}

Point3D
operator+(const Point3D &p1, const Point3D &p2)
{ 
  return Point3D(p1[0] + p2[0], p1[1] + p2[1], p1[2] + p2[2]);
}

Point3D
operator-(const Point3D &p1, const Point3D &p2)
{ 
  return ((-1 * p2) + p1);
}

Point3D
operator*(const Point3D &p1, const double scalar)
{ 
  return Point3D(p1[0] * scalar, p1[1] * scalar, p1[2] * scalar);
}

Point3D
operator*(const double scalar, const Point3D &p1)
{ 
  return (p1 * scalar);
}

bool
operator==(const Point3D& p1, const Point3D& p2)
{
  return ( p1.x == p2.x && p1.y == p2.y && p1.z == p2.z);
}


Point3D
operator*(const Point3D &p1, const Point3D &p2)
{ 
  return Point3D((double) (p1[0] * p2[0]),
		 (double) (p1[1] * p2[1]),
		 (double) (p1[2] * p2[2]));
}
