/** @file point.h */

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                       /
// Projet : Gnrateur de cylindres gnraliss                                          /
//                                                                                       /
// Nom du fichier : point.h                                                              /
//                                                                                       /
// Creation : 20/11/2005                                                                 /
//                                                                                       /
// Description : La classe Point3D permet de spcifier des points dans l'espace 3D.      / 
//                                                                                       /
// Auteurs : - Guitteny Fabrice                                                          /
//           - Idiart Baptiste                                                           /
//           - Le Goff Erwan                                                             /
//                                                                                       /
//////////////////////////////////////////////////////////////////////////////////////////

#ifndef POINT_3D_H
#define POINT_3D_H

#include<iostream>

using namespace std;


/** La classe Point3D permet de spcifier des points dans l'espace R3.
 *
 * @author Guitteny Fabrice
 * @author Idiart Baptiste
 * @author Le Goff Erwan
 */

///////////////////////////////////////////////////////////////////////////////////////////
//  Classe Point3D                                                                       //
///////////////////////////////////////////////////////////////////////////////////////////
    
class Point3D {
  
  /** operateur << (affichage des coordonnes du point)
   * @param os flux de sortie
   * @param pt point dont on veut afficher les coordonnes
   * @return flux
   */
  friend std::ostream& operator<<(std::ostream& os,const Point3D& pt);
  
  /** operateur + (addition de 2 points)
   * @param p1 point
   * @param p2 point
   * @return Point3D p1 + p2
   */
  friend Point3D operator+(const Point3D& p1,const Point3D& p2);

  /** operateur - (soustraction de 2 points)
   * @param p1 point
   * @param p2 point
   * @return Point3D p1 - p2
   */
  friend Point3D operator-(const Point3D& p1,const Point3D& p2);
  
  /** operateur * (multiplication avec un scalaire)
   * @param p1 point
   * @param scalaire multiplicateur
   * @return Point3D p1 * scalaire
   */
  friend Point3D operator*(const Point3D& p1,const double scalaire);
  
  /** operateur * (multiplication avec un scalaire)
   * @param scalaire multiplicateur
   * @param p1 point
   * @return Point3D scalaire * p1
   */
  friend Point3D operator*(const double scalaire, const Point3D& p1);
  
  /** operateur == (galit de 2 points)
   * @param p1 point
   * @param p2 point
   * @return bool p1 == p2
   */
  friend bool operator==(const Point3D& p1, const Point3D& p2);
  
  /** operateur * (multiplication de 2 points)
   * @param p1 point
   * @param p2 point
   * @return Point3D p1 * p2
   */
  friend Point3D operator*(const Point3D& p1, const Point3D& p2);


 private:

  double x, y, z;           // coordonnes 3D d'un point

 public:

  /** constructeur par copie
   *@param point Point3D
   */
  Point3D(const Point3D &point);

  /** constructeur 
   *@param xx abcisse
   *@param yy ordonne
   *@param zz hauteur
   */
  Point3D(const double xx=0,const double yy=0,const double zz=0);

  /** accesseur
   * @param i dimension voulue (0 : x, 1 : y, 2 : z)
   * @return x, y ou z (selon i)
   */
  double operator[](int i) const;

  /** modificateur
   * @param i dimension voulue (0 : x, 1 : y, 2 : z)
   * @return x, y ou z (selon i)
   */
  double& operator[](int i);

  /** accesseur
   * @param i dimension voulue (0 : x, 1 : y, 2 : z)
   * @return x, y ou z (selon i)
   */
  double& getCoordinate(int i);

  /** calcul de la distance entre 2 points
   * @param point point dont on veut calculer la distance
   */
  double distance (Point3D &point);

  /** operateur *= (multiplication par un scalaire)
   * @param scalaire multiplicateur
   * @return Point3D
   */
  Point3D& operator*=(const double scalaire);

  /** operateur += (addition de 2 points)
   * @param point autre point
   * @return Point3D
   */
  Point3D& operator+=(const Point3D& point);

  /** operateur -= (soustraction de 2 points)
   * @param point autre point
   * @return Point3D
   */
  Point3D& operator-=(const Point3D& point);
};

std::ostream& operator<<(std::ostream &os,const Point3D& point);

Point3D operator+(const Point3D& p1,const Point3D& p2);

Point3D operator-(const Point3D& p1,const Point3D& p2);

Point3D operator*(const Point3D& p1,const double scalaire);

Point3D operator*(const double scalaire, const Point3D& p1);

bool operator==(const Point3D& p1, const Point3D& p2);

Point3D operator*(const Point3D& p1, const Point3D& p2);

#endif
