/** @file primitive.h */

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                       /
// Projet : Gnrateur de cylindres gnraliss                                          /
//                                                                                       /
// Nom du fichier : primitive.h                                                          /
//                                                                                       /
// Creation : 20/11/2005                                                                 /
//                                                                                       /
// Description : La classe Primitive permet de spcifier et de dessiner une primitive.   / 
//                                                                                       /
// Auteurs : - Guitteny Fabrice                                                          /
//           - Idiart Baptiste                                                           /
//           - Le Goff Erwan                                                             /
//                                                                                       /
//////////////////////////////////////////////////////////////////////////////////////////

#ifndef PRIMITIVE_H
#define PRIMITIVE_H

#include<iostream>
#include <qpainter.h>
#include <qdom.h> 
#include <vector>

#include "point.h"
#include "../interface/utilities.h"

using namespace std;

// types de primitives
const int SEGMENT = 0;
const int CIRCLE = 1;
const int BEZIER = 2;
const int BSPLINE = 3;

// valeurs des chantillonage
const int LOW_SAMPLE = 30;
const int NORMAL_SAMPLE = 15;
const int ACCURATE_SAMPLE = 9;

/** La classe Primitive permet de spcifier et de dessiner une primitive.
 *
 * @author Guitteny Fabrice
 * @author Idiart Baptiste
 * @author Le Goff Erwan
 */

///////////////////////////////////////////////////////////////////////////////////////////
//  Classe Primitive                                                                     //
///////////////////////////////////////////////////////////////////////////////////////////

class Primitive 
{
  public :

    /** liste des points dfinissant la primitive */
    vector<Point3D*> pointsList;

  /** liste des points pour l'chantillonage */
  vector<Point3D*> samplesPointsList;

  /** nombre d'chantillons */
  double nbSamples;

  /** type de la primitive
   * SEGMENT, CERCLE, BEZIER ou BSPLINE
   */
  int type;

  /** nombre de points dfinissant la primitive */
  int nbPoints() {return pointsList.size();}

  /** destructeur */
  virtual ~Primitive() {clearVector(pointsList); clearVector(samplesPointsList);}

   /** accesseur : valeur de l'chantillonage
   * @return nombre d'chantillons
   */
  inline double getNbSamples() {return nbSamples;}

  /** modificateur : valeur de l'chantillonage
   * @param sample nouvelle valeur de l'chantillonage
   */
  inline void setNbSamples(double sample)
    {
      if (type == CIRCLE)
	nbSamples = sample / 3;
      else
	nbSamples = sample;
    }

  /** mthode d'affichage de la primitive
   * @param paint affichage dans un QPainter
   */
  virtual void display(QPainter & paint) = 0;

  /** changement des points dfinissant une primitive
   * @param refPoint point  modifier
   * @param modifPoint nouvelle valeur du point
   */
  virtual void setPrimitive(QPoint & refPoint, QPoint & modifPoint) = 0;

  /** ecrit la primitive dans un fichier xml
   * @param elem balise courante dans le fichier XML
   */
  virtual void toXML(QDomElement &elem) = 0 ;
};

#endif
