#include "segment.h"

// constructeurs

Segment::Segment()
{
  type = SEGMENT;
  nbSamples = NORMAL_SAMPLE;
}

Segment::Segment(Point3D * pA, Point3D * pB) 
{
  pointsList.push_back(pA);
  pointsList.push_back(pB);
  type = SEGMENT;
  nbSamples = NORMAL_SAMPLE;
}

Segment::Segment(const Segment& seg) : Primitive(seg)
{
  Point3D * pA = seg.pointsList[0];
  Point3D * pB = seg.pointsList[1];
  Segment(pA, pB);
}


// accesseurs
Point3D *
Segment::getPointA() 
{
  return pointsList[0];
}

Point3D *
Segment::getPointB() 
{
  return pointsList[1];
}



// modificateurs

void
Segment::setPointA(Point3D * pA)
{
  pointsList[0] = pA;
}

void
Segment::setPointB(Point3D * pB)
{
  pointsList[1] = pB;
}

// mthode tudiant le signe d'un entier
// renvoie -1 si < 0
//          0 si = 0
//          1 si > 0
static int
sign(int diff)
{
  if (diff < 0)
    return -1;
  else
    if (diff == 0)
      return 0;
    else
      return 1;
}

// mthode d'affichage d'un segment (Bresenham)
void
Segment::display(QPainter& paint)
{
  // on rcupre les coordonnes des extrmits du segment 
  int x1 = (int)(pointsList[0]->getCoordinate(0));
  int y1 = (int)(pointsList[0]->getCoordinate(1));
  int x2 = (int)(pointsList[1]->getCoordinate(0));
  int y2 = (int)(pointsList[1]->getCoordinate(1));
  
  int x = x1;
  int y = y1;
  
  int dx = abs(x2 - x1);
  int dy = abs(y2 - y1);

  int s1 = sign(x2 - x1);
  int s2 = sign(y2 - y1);

  bool a_change = false;

  if (dy > dx)
    {
      int temp = dx;
      dx = dy;
      dy = temp;
      a_change = true;
    }

  int e = 2 * dy - dx;
  
  int temp = 0;

  samplesPointsList.clear();

  for (int i = 0; i < dx - 1; i++)
    {
      if (temp >= nbSamples)
	{
	  samplesPointsList.push_back(new Point3D(x,y,0));

	  temp = 0;
	}
      temp++;

      paint.drawPoint(x, y);
      while (e > 0)
	{
	  if (a_change)
	    x += s1;
	  else
	    y += s2;
	  e -= 2 * dx;
	}
      if (a_change)
	y += s2;
      else
	x += s1;
      e += 2 * dy;
    }
  
  samplesPointsList.push_back(new Point3D(*pointsList[1]));

}


// changement des points dfinissant un segment
void
Segment::setPrimitive(QPoint & refPoint, QPoint & modifPoint)
{
  Point3D * p = qPointToPoint3D(refPoint);
  if (*p == *getPointA())
    setPointA(qPointToPoint3D(modifPoint));
  if (*p == *getPointB())
    setPointB(qPointToPoint3D(modifPoint));
}



// ecrit la primitive dans un fichier xml
void
Segment::toXML(QDomElement &elem)
{
  elem.setAttribute( "Ax",  (*getPointA())[0] );  
  elem.setAttribute( "Ay",  (*getPointA())[1] );
  elem.setAttribute( "Bx",  (*getPointB())[0] );
  elem.setAttribute( "By",  (*getPointB())[1] );
 
}


