#include "vector3D.h"

// Constructeurs
Vector3D::Vector3D(const Vector3D &vector)
{
  for (int i = 0; i < 3; i++)
    vect[i] = vector[i];
}

Vector3D::Vector3D(const double xx, const double yy, const double zz)
{
  vect[0] = xx;
  vect[1] = yy;
  vect[2] = zz;
}

Vector3D::Vector3D(const Point3D &p1, const Point3D &p2)
{
  // p1 -> point de dpart	P2 -> point d'arrive
  vect[0] = p2[0] - p1[0];
  vect[1] = p2[1] - p1[1];
  vect[2] = p2[2] - p1[2];
}


// Accesseurs
double
Vector3D::operator[](int i) const
{
  return vect[i];
}

double&
Vector3D::operator[](int i)
{
  return vect[i];
}

// norme d'un vecteur
double
Vector3D::norm()
{
  return (double) sqrt(vect[0] * vect[0] + vect[1] * vect[1] + vect[2] * vect[2]);
}
  
// normalisation d'un vecteur
Vector3D
Vector3D::normalize()
{  
  double vectNorm = norm();
  return Vector3D(vect[0] / vectNorm,
		  vect[1] / vectNorm,
		  vect[2] / vectNorm);
}


// produit scalaire
double
Vector3D::operator * (const Vector3D &vector)
{
  return (vect[0] * vector[0] + vect[1] * vector[1] + vect[2] * vector[2]);
}

// produit vectoriel
Vector3D
Vector3D::operator ^ (const Vector3D &vector)
{
  Vector3D vNormal;
  vNormal[0] = ((vect[1] * vector[2]) - (vect[2] * vector[1]));
  vNormal[1] = ((vect[2] * vector[0]) - (vect[0] * vector[2]));
  vNormal[2] = ((vect[0] * vector[1]) - (vect[1] * vector[0]));
  return vNormal;                                     
}

Vector3D&
Vector3D::operator*=(const double scalar)
{
  for (int i = 0; i < 3; i++)
    vect[i] *= scalar;
  return *this;
}

Vector3D&
Vector3D::operator+=(const Vector3D &vector)
{
  for (int i = 0; i < 3; i++)
    vect[i] += vector[i];
  return *this;
}

Vector3D&
Vector3D::operator-=(const Vector3D &vector)
{
  for (int i = 0; i < 3; i++)
    vect[i] -= vector[i];
  return *this;
}

// Operateurs externes

std::ostream&
operator<<(std::ostream &os,const Vector3D &vect)
{
  os << "[" << vect[0] << "," <<  vect[1] <<  "," <<  vect[2] << "]";
  return os;
}

Vector3D
operator+(const Vector3D &v1,const Vector3D &v2)
{ 
  return Vector3D(v1[0] + v2[0],
		  v1[1] + v2[1],
		  v1[2] + v2[2]);
}

Vector3D
operator-(const Vector3D &v1,const Vector3D &v2)
{ 
  return ((-1 * v2) + v1);
}

Vector3D
operator*(const Vector3D &v1,const double scalar)
{ 
  return Vector3D(v1[0] * scalar,
		  v1[1] * scalar,
		  v1[2] * scalar);
}

Vector3D
operator*(const double scalar, const Vector3D &v1)
{ 
  return (v1 * scalar);
}

