/** @file vector3D.h */

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                       /
// Projet : Gnrateur de cylindres gnraliss                                          /
//                                                                                       /
// Nom du fichier : vector3D.h                                                           /
//                                                                                       /
// Creation : 20/11/2005                                                                 /
//                                                                                       /
// Description : La classe Vector3D permet de spcifier des vecteurs en 3D.              / 
//                                                                                       /
// Auteurs : - Guitteny Fabrice                                                          /
//           - Idiart Baptiste                                                           /
//           - Le Goff Erwan                                                             /
//                                                                                       /
//////////////////////////////////////////////////////////////////////////////////////////

#ifndef VECTOR_3D_H
#define VECTOR_3D_H

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <iostream>
#include "point.h"

/** La classe Vector3D permet de spcifier des vecteurs en 3D.                       
 *
 * @author Guitteny Fabrice
 * @author Idiart Baptiste
 * @author Le Goff Erwan
 */

///////////////////////////////////////////////////////////////////////////////////////////
//  Classe Vector3D                                                                      //
///////////////////////////////////////////////////////////////////////////////////////////

class Vector3D {

  /** operateur << (affichage des coordonnes du vecteur)
   * @param os flux de sortie
   * @param vect vecteur dont on veut afficher les coordonnes
   * @return flux
   */
  friend std::ostream& operator<<(std::ostream &os,const Vector3D &vect);

  /** operateur + (addition de 2 vecteurs)
   * @param v1 vecteur
   * @param v2 vecteur
   * @return Vector3D v1 + v2
   */
  friend Vector3D operator+(const Vector3D &v1,const Vector3D &v2);

  /** operateur - (soustraction de 2 vecteurs)
   * @param v1 vecteur
   * @param v2 vecteur
   * @return Vector3D v1 - v2
   */
  friend Vector3D operator-(const Vector3D &v1,const Vector3D &v2);
  
  /** operateur * (multiplication avec un scalaire)
   * @param v1 vecteur
   * @param scalar multiplicateur
   * @return Vector3D v1 * scalar
   */
  friend Vector3D operator*(const Vector3D &v1,const double scalar);
  
  /** operateur * (multiplication avec un scalaire)
   * @param v1 vecteur
   * @param scalar multiplicateur
   * @return Vector3D scalar * v1
   */
  friend Vector3D operator*(const double scalar, const Vector3D &v1);
 

 private:

  /** coordonnes du vecteur */
  double vect[3];


 public:

  /** constructeur par copie
   * @param vector Vector3D
   */
  Vector3D(const Vector3D &vector);

  /** constructeur : spcification des coordonnes
   * @param xx abcisse
   * @param yy ordonne
   * @param zz hauteur
   */
  Vector3D(const double xx = 0, const double yy = 0, const double zz = 0);

  /** constructeur : spcification des points extrmits
   * @param p1 point extrmit 1
   * @param p2 point extrmit 2
   */
  Vector3D(const Point3D &p1, const Point3D &p2);

  /** accesseur
   * @param i dimension voulue (0 : x, 1 : y, 2 : z)
   * @return x, y ou z (selon i)
   */
  double operator[](int i) const;

  /** modificateur
   * @param i dimension voulue (0 : x, 1 : y, 2 : z)
   * @return x, y ou z (selon i)
   */
  double& operator[](int i);

  /** norme d'un vecteur
   * @return norme du vecteur
   */
  double norm();
  
  /** normalisation d'un vecteur
   * @return vecteur normails
   */
  Vector3D normalize();


  /** operateur * (produit scalaire de 2 vecteurs)
   * @param vector autre vecteur
   * @return produit scalaire
   */
  double operator * (const Vector3D &vector);

  /** operateur ^ (produit vectoriel de 2 vecteurs)
   * @param vector autre vecteur
   * @return produit vectoriel
   */
  Vector3D operator ^ (const Vector3D &vector);

  /** operateur *= (multiplication par un scalaire)
   * @param scalar multiplicateur
   * @return Vector3D
   */
  Vector3D& operator*=(const double scalar);

  /** operateur += (addition de 2 vecteurs)
   * @param vector autre vector
   * @return Vector3D
   */
  Vector3D& operator+=(const Vector3D &vector);

  /** operateur -= (soustraction de 2 vecteurs)
   * @param vector autre vector
   * @return Vector3D
   */
  Vector3D& operator-=(const Vector3D &vector);
};

std::ostream& operator<<(std::ostream&,const Vector3D&);
Vector3D operator+(const Vector3D&,const Vector3D&);
Vector3D operator-(const Vector3D&,const Vector3D&);
Vector3D operator*(const double ,const Vector3D&);
Vector3D operator*(const Vector3D&,const double);

#endif
