/** @file parser.h */

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                       /
// Projet : Gnrateur de cylindres gnraliss                                          /
//                                                                                       /
// Nom du fichier : parser.h                                                             /
//                                                                                       /
// Creation : 20/11/2005                                                                 /
//                                                                                       /
// Description : La classe Parser est abstraite. Elle regoupe les caractriques          /
//               minimum et fonctionnelles pour un parser.                               /
//               VrmlParser et PgnParser hrite de Parser.                               /
//                                                                                       /
//                                                                                       /            
// Auteurs : - Guitteny Fabrice                                                          /
//           - Idiart Baptiste                                                           /
//           - Le Goff Erwan                                                             /
//                                                                                       /
//////////////////////////////////////////////////////////////////////////////////////////


#ifndef PARSER_H
#define PARSER_H

#include <iostream>
#include <string.h>
#include <stdio.h>
#include <fstream>
#include <math.h>
#include <vector>

#include "../primitives/point.h"
#include "../interface/utilities.h"
#include "utils.h"



using namespace std;



/**
 *
 * La classe Parser est abstraite. Elle regoupe les caractriques
 * minimum et fonctionnelles pour un parser.
 * VrmlParser et PgnParser hrite de Parser.
 *
 *  @author Guitteny Fabrice
 *  @author Idiart Baptiste
 *  @author Le Goff Erwan
 */
///////////////////////////////////////////////////////////////////////////////////////////
//  Classe Parser                                                                        //
///////////////////////////////////////////////////////////////////////////////////////////


class Parser
{

protected :
  /** @param nbPoints nombre de points
   */
  int nbPoints;
  /** @param nbPolys nombre de polygones
   */
  int nbPolys;
  /** @param pointsList liste de points du model
   */
  vector<Point3D*> pointsList;
  /** @param referencesList liste de rfrences du model
   */
  vector<int> referencesList;

public : 
  /** Positionne les paramtres ncssaires pour l'export d'un modele
   *@param nbPoints nombre de points
   *@param nbPolys nombre de polygones
   *@param pointsList liste de points du model
   *@param referencesList liste de rfrences du model
   */
  virtual void setParameters(const int &nbPoints, const int &nbPolys, const vector<Point3D*> &pointsList, const vector<int> &referencesList);
  /** destructeur virtuel
   */
  virtual ~Parser();
  /** @return nombre de points
   */
  inline virtual int getNbPoints(){return nbPoints;}
  /** @return nombre de polygones
   */
  inline virtual int getNbPolys(){return nbPolys;}
  /** @return pointsList liste de points du model
   */
  inline virtual vector<Point3D*> getPointsList(){return pointsList;}
  /** @return referencesList liste de rfrences du model
   */
  inline virtual vector<int> getReferencesList(){return referencesList;}
};

#endif
