#include "window3D.h"

// constructeurs
Window3D::Window3D( QWidget *parent, char *name, int timeInterval) : QGLWidget( parent, name ) 
{
 timer = new QTimer( this );
 if( timeInterval == 0 )
		timer = 0;
	else
	{
		connect( timer, SIGNAL(timeout()), this, SLOT(timeOutSlot()) );
		timer->start( timeInterval );
	}
  setCaption(name);//nom de la fenetre Widget
  //mouvements
  angle[0]=0.0;angle[1]=0.0;angle[2]=0.0;
  tra[0]=0; tra[1]=0; tra[2]=0;
  zoom=1.0;
  //lumiere
  //Pour pouvoir observer le modle sous tous ces angles, deux lumieres sont utilises
  light1Position[0]=0.0;light1Position[1]=400.0;light1Position[2]=0.0;light1Position[3]=1.0;
  light2Position[0]=0.0;light2Position[1]=-400.0;light2Position[2]=0.0;light2Position[3]=1.0;
  ambientLight[0]=0.5;ambientLight[1]=0.6;ambientLight[2]=0.8;ambientLight[3]=1.0;
  diffuseLight[0]=0.8;diffuseLight[1]=0.8;diffuseLight[2]=0.8;diffuseLight[3]=1.0;
  specularLight[0]=0.4;specularLight[1]=0.5;specularLight[2]=0.6;specularLight[3]=1.0;
  nbPoints = 0;//nombre de points
  nbPolys = 0;//nombre de polygones
  model = 0;//numro du model de la displaylist
}


// chargement d'un modle en mmoire
// gnration des normales puis cration d'un modle prcompil
int Window3D::load(const int &nbPts, const int &nbPls, const vector<Point3D*> &PtsList, const vector<int> &RefList)
{
  //suppresion du modele existant
  glDeleteLists(model, 1 );
  glMatrixMode (GL_MODELVIEW);
  glLoadIdentity ();
  nbPoints = nbPts;
  nbPolys = nbPls;
  pointsList = PtsList;
  referencesList = RefList;

  for (int i = 0; i < nbPolys * 3; i += 3)
    {
      Point3D pA = *pointsList[referencesList[i]];
      Point3D pC = *pointsList[referencesList[i + 1]];
      Point3D pB = *pointsList[referencesList[i + 2]];
      
      Vector3D v1 = Vector3D(pA, pC);
      Vector3D v2 = Vector3D(pA, pB);
      
	  //la normale d'une facette doit tre dirige vers l'extrieur
      Vector3D  normalFacet = v2^v1;
      
      normalFacet.normalize();
      
      normalsList.push_back(new Vector3D(normalFacet));      
    }

  //modele prcompil
  model = drawModel();
  return 1;
}


//suppression du modle
void Window3D::clear()
{
  nbPoints = 0;
  nbPolys = 0;
  pointsList.clear();
  referencesList.clear();
  normalsList.clear();
  glDeleteLists(model, 1);
}

//initialisation de la fentre OpenGL
void Window3D:: initializeGL()
{
  glClearColor (0.0, 0.0, 0.0, 0.0);
  glClear (GL_COLOR_BUFFER_BIT);
  
  glViewport (0, 0, WIDTHOGL, HEIGHTOGL);
      
  glMatrixMode (GL_PROJECTION);
  glLoadIdentity ();
  glOrtho(-1.1, 1.1, -1.1, 1.1, -10, 10);
      
  glMatrixMode (GL_MODELVIEW);
  glLoadIdentity ();
  setAutoBufferSwap (true);
      
  glEnable(GL_DEPTH_TEST);

  glEnable(GL_NORMALIZE);


  //Paramtres des 2 lumires
  glShadeModel(GL_SMOOTH);
  glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER,GL_TRUE);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_LIGHT1);
  glLightfv(GL_LIGHT0,GL_AMBIENT,ambientLight);
  glLightfv(GL_LIGHT0,GL_DIFFUSE,diffuseLight);
  glLightfv(GL_LIGHT0,GL_SPECULAR,specularLight);
  glLightfv(GL_LIGHT1,GL_AMBIENT,ambientLight);
  glLightfv(GL_LIGHT1,GL_DIFFUSE,diffuseLight);
  glLightfv(GL_LIGHT1,GL_SPECULAR,specularLight);
  
  
}

      
//en cas de redimensionnemment de la fentre
void Window3D::resizeGL( int width, int height )
{
  height = height?height:1;
      
  glViewport( 0, 0, (GLint)width, (GLint)height );
      
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,100.0f);
      
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}

//matrice de rotation
void 
Window3D::rotate ()
{
  glRotatef (angle[1], 1.0, 0.0, 0.0);
  glRotatef (angle[0], 0.0, 1.0, 0.0);
  glRotatef (angle[2], 0.0, 0.0, 1.0);
}

//matrice de translation
void 
Window3D::translate ()
{
  glTranslatef (tra [0], tra [1], tra [2]);
}

//matrice de mise  l'chelle
void 
Window3D::scale ()
{
  glScalef (zoom, zoom, zoom);
}
 

//dessin dans la fentre des 3 axes du repre 3D
void Window3D::drawAxis()
{
  glDisable(GL_LIGHTING);
  glBegin(GL_LINES);
  glColor3f(0.0,1.0,0.0);
  glVertex3f(0.0,0.0,0.0);
  glVertex3f(5.0,0.0,0.0);
  glColor3f(0.0,0.0,1.0);


  glVertex3f(0.0,0.0,0.0);
  glVertex3f(0.0,5.0,0.0);
  glColor3f(1.0,0.0,0.0);

  glVertex3f(0.0,0.0,0.0);
  glVertex3f(0.0,0.0,5.0);
  glEnd();
  glEnable(GL_LIGHTING);
}


//Fonction d'affichage
void Window3D::paintGL()
{
  glClear (GL_COLOR_BUFFER_BIT| GL_DEPTH_BUFFER_BIT);
   drawAxis(); 
  glMatrixMode (GL_PROJECTION);
  glLoadIdentity (); 
  gluPerspective (90, 1, 0.1, 1000);

  glMatrixMode (GL_MODELVIEW);
  glLoadIdentity ();
  gluLookAt(0.0,0.0,50.0,0.0,0.0,-100.0,0.0,1.0,0.0);
  glLightfv (GL_LIGHT0, GL_POSITION, light1Position);
  glLightfv (GL_LIGHT1, GL_POSITION, light2Position);

  translate ();
  rotate ();
  scale();

  //appel du modele prcompil
  glCallList(model);

  //affichage en haut  gauche du nombre de triangle et du facteur de mise  l'chelle
  glDisable(GL_LIGHTING);
  glColor3f(1.0,0.0,0.0);
  stringstream infos;
  infos << "Triangles : " << nbPolys << "    Scale : x" << zoom ;
  renderText ( 5, 15, infos.str());
  glEnable(GL_LIGHTING);
  
  glFlush();
}


//cration du modle prcompil
GLuint Window3D:: drawModel()
{
  GLuint list;
  list = glGenLists(1);

  glNewList(list, GL_COMPILE);
  glColor3f(0.0,0.6,0.8);
  glBegin(GL_TRIANGLES);
  //Les polygones sont uniquement triangulaires donc une facette est rfrencie 3 points de l'espace
  //Ainsi le nombre de rfrences est 3 fois plus important que le nombre de polygones
  for(int i=0;i<3*nbPolys;i+=3)
    {
      glNormal3f ((*normalsList[i/3])[0],(*normalsList[i/3])[1],(*normalsList[i/3])[2]);

      glVertex3f( (pointsList[referencesList[i]])->getCoordinate(0), (pointsList[referencesList[i]])->getCoordinate(1), (pointsList[referencesList[i]])->getCoordinate(2));
      
      glVertex3f( (pointsList[referencesList[i+1]])->getCoordinate(0), (pointsList[referencesList[i+1]])->getCoordinate(1), (pointsList[referencesList[i+1]])->getCoordinate(2));
      
      glVertex3f( (pointsList[referencesList[i+2]])->getCoordinate(0), (pointsList[referencesList[i+2]])->getCoordinate(1), (pointsList[referencesList[i+2]])->getCoordinate(2)); 
    
    }
  glEnd();
  glEndList();

  return list;
}
  
//Expiration du temporisateur
//On lance la fonction d'affichage
void Window3D::timeOut()
{
  updateGL();
}
  
  
//Action de l'utlisateur sur les boutons de la souris
void Window3D::mousePressEvent(QMouseEvent *event)
{    
  button = event->button();
  oldX =  event->x();
  oldY =  event->y();
}

//Action de l'utlisateur sur la molette de la souris
void Window3D::wheelEvent( QWheelEvent * e )
{    
	//la molette de la souris permet de modifier le facteur de mise  l'chelle
  if((e->delta() > 0) && (zoom < 100 * ZOO_INC))
      zoom+=ZOO_INC;
  if((e->delta() < 0) && (zoom > ZOO_INC))
	zoom -= ZOO_INC;
}

//Dplacements de la souris
void Window3D::mouseMoveEvent(QMouseEvent *event){
  float dx = event->x() - oldX; 
  float dy = event->y() - oldY;
      
  if (button == Qt::LeftButton) {
	//Le bouton gauche permet les rotations selon l'axe x et y suivant le dplacement
    angle[0] += dx;
    angle[1] += dy;

  }
  else if (button == Qt::RightButton) {
	//le bouton droit permet les translations dans la profondeur (axe z)
    tra [2] += (dy * TRA_INC);
    oldY =  event->y();
  }
  else if (button == Qt::MidButton) {
	//le bouton du milieu permet les translations selon l'axe x et l'axe y 
    tra [0] += dx*TRA_INC;
    tra [1] -= dy*TRA_INC;
  }
  oldX = event->x() ;
  oldY = event->y();   
}
  
  
//Action de l'utilisateur sur une touche du clavier
void Window3D::keyPressEvent( QKeyEvent *e )
{
  int key = e->key();
  switch (key)
    {
    case 4112 : //touche 'home' position de dpart
      angle[0]=0.0; angle[1]=0.0; angle[2]=0.0;
      tra[0]=0.0; tra[1]=0.0; tra[2]=0.0;
      zoom=1.0;
      break;
    case 4118 : //touche 'Page Down' augmentation du zoom
      if(zoom < 100 * ZOO_INC)
	zoom += ZOO_INC;
      break;
    case 4119 : //touche 'Page Up' diminution du zoom
      if(zoom > 2*ZOO_INC)
	zoom -= ZOO_INC;
      break;
    case 88 : // touche 'x' affichage en mode plein
      glEnable(GL_LIGHTING);
      glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
      break;
    case 87 : //touche 'w' affichage en mode fil de fer 
      glDisable(GL_LIGHTING);
      glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
      break;
    case 67 : // touche 'c' affichage en mode sommets seuls
      glDisable(GL_LIGHTING);
      glPolygonMode(GL_FRONT_AND_BACK,GL_POINT);
      break;
    }
    
  updateGL();
}

//Export du modele
//Cette fonction est appele pour exporter un modle en Vrml1.0 ou en PGN
int Window3D:: exportOGL(int &nbPts,int &nbPls, vector<Point3D*> &PtsList, vector<int> &RefList)
{
  nbPts = nbPoints;
  nbPls = nbPolys;
  PtsList = pointsList;
  RefList = referencesList;
  
  return 1;
}

//destructeur de Window3D
Window3D::~Window3D()
{
  pointsList.clear();
  referencesList.clear();
  glDeleteLists(model, 1);
}

//Action de fermeture de la fentre
//Le corps de cette fonction est vide pour ne pas autoriser la fermeture de la fentre OpenGL
void
Window3D::closeEvent(QCloseEvent * ) {}


//Lorsque le timer arrive  expiration
void
Window3D::timeOutSlot()
{
	timeOut();
}

