
#include "modelml.h"

// Constructeur de Modelml
// L'entete du document sera de type <!DOCTYPE ModelMG>

Modelml::Modelml() : QDomDocument("ModelMG"){}



void Modelml::save(const string &output ,const vector<Primitive*> &primProfile,const vector<Primitive*> &primSection,const vector<Primitive*> &primAxis)
{
  
  QFile file(output);
  if( !file.open( IO_WriteOnly ) )
    exit(-1); // quitte si pas en accssible en criture
  QTextStream ts( &file );
  //balise principale  
  QDomElement root = createElement( "model" );
  //balises secondaires (une pour chaque fenetre)
  QDomElement profile = createElement( "profil" );
  QDomElement section = createElement( "section" );
  QDomElement axis = createElement( "axe" );

  appendChild( root );//ajout de la racine au doctype ModelMG
  root.appendChild(profile);//puis cration d'une balise vide pour profil
  root.appendChild(section);//pour section
  root.appendChild(axis);//pour axe

  appendElem(primProfile,profile);//remplissage de la balise profil avec l'ensemble de primitives de la fentre
  appendElem(primSection,section);//remplissage de la balise section avec l'ensemble de primitives de la fentre
  appendElem(primAxis,axis);//remplissage de la balise axe avec l'ensemble de primitives de la fentre

  //Ecriture dans le fichier output
  ts << toString();
  file.close();//fermeture du fichier
}

void Modelml::appendElem(const vector<Primitive*> &primitivesList, QDomElement &elem){
  
  QDomElement  sselem;//sous lment de la balise elem
  for(unsigned int i = 0 ; i<primitivesList.size() ; i++)
    {
    switch(primitivesList[i]->type){
      
    case SEGMENT :
      sselem = createElement( "segment" );
      primitivesList[i]->toXML(sselem);//mise en forme des donnes de la primitive au format Xml
      elem.appendChild(sselem);//ajout du sous lment  la balise elem
      break;
      
    case CIRCLE :
      sselem = createElement( "cercle" );
      primitivesList[i]->toXML(sselem);//mise en forme des donnes de la primitive au format Xml
      elem.appendChild(sselem);//ajout du sous lment  la balise elem
      break;
      
    case BEZIER :
      sselem = createElement( "bezier" );

      primitivesList[i]->toXML(sselem);//mise en forme des donnes de la primitive au format Xml
      elem.appendChild(sselem);//ajout du sous lment  la balise elem
      break;
    case BSPLINE :
      sselem = createElement( "bspline" );
      primitivesList[i]->toXML(sselem);//mise en forme des donnes de la primitive au format Xml
      elem.appendChild(sselem);//ajout du sous lment  la balise elem
      break;
    default : cout<<"non implement\n";exit(-1);break;
    
    }
  }
}


vector<vector<Primitive*> > Modelml::load(const string input)
{
  //verification de la validit du fichier input
  QFile file(input);
  if( !file.open( IO_ReadOnly ) )
    exit(-1);//le fichier doit tre au moins en lecture seule
  //chargement des informations dans l'arbre Dom
  if( !setContent( &file ) )
    {//le fichier Xml doit tre un fichier de type <!DOCTYPE ModelMG>
      file.close();
      exit(-2);
    }
  file.close();

  // A ce stade le fichier est ferm puisque toutes les donnes ont t extraites
  // dans l'arbre Dom de l'objet Modelml

  vector<Primitive*> primProfile; //primitives de la balise profil
  vector<Primitive*> primSection;//primitives de la balise section
  vector<Primitive*> primAxis;//primitives de la balise axe

  vector<vector<Primitive*> > primitivesList; //vecteur des 3 balises
 
  QDomElement root = documentElement();
  if( root.tagName() != "model" )
    exit(-3);
  QDomNode model = root.firstChild();//la premiere balise est "model"
  while( !model.isNull() )
    {
      //parcours des fils de "model"
      //3 fils possibles : profile, section, axe
      //on cre ensuite l'ensemble des primitives dans le vecteur concern
      QDomElement e = model.toElement();
      if( !e.isNull() ){
	if( e.tagName() == "profil" ){
	  loadElem(e,primProfile);primitivesList.push_back(primProfile);}
	if( e.tagName() == "section" ){
	  loadElem(e,primSection); primitivesList.push_back(primSection);}
	if( e.tagName() == "axe" ){
	  loadElem(e,primAxis);primitivesList.push_back(primAxis);}
      }
      model = model.nextSibling();//passage au fils suivant
    }
   return primitivesList;
}


void Modelml::loadElem(QDomElement &e,vector<Primitive*> &primitivesList)
{
  QDomNode node = e.firstChild();
  Primitive *p;
  while(!node.isNull())
    {
      QDomElement attr = node.toElement();
      p = loadAttribut(attr);//cration d'une primitive  partir des donnes de l'arbre Dom
      primitivesList.push_back(p);//ajout  l'ensemble primitivesList de la primitive p
      node = node.nextSibling();//passage au fils suivant
    }  
}

Primitive* Modelml::loadAttribut(const QDomElement& attr)
{
  Primitive *p = NULL;
  if( !attr.isNull() )
    {
      if(attr.tagName() == "segment" )
	{//cration d'un segment  partir de deux points extrmits
	  Point3D *a =new Point3D( atof(attr.attribute( "Ax", "" )),atof(attr.attribute( "Ay", "" )));
	  Point3D *b =new Point3D( atof(attr.attribute( "Bx", "" )),atof(attr.attribute( "By", "" )));
	  p= new Segment(a,b);
	}
      
      if(attr.tagName() == "cercle" )
	{//cration d'un cercle  partir du Point centre et du rayon
	  Point3D *a =new Point3D( atof(attr.attribute( "Ax", "" )),atof(attr.attribute( "Ay", "" )));
	  // double ray = atof(attr.attribute( "rayon", "" ));
	  Point3D *b =new Point3D( atof(attr.attribute( "Bx", "" )),atof(attr.attribute( "By", "" )));
	  p = new Circle(a,b);
  	}

      if(attr.tagName() == "bezier" )
	{//cration d'une courbe de Bezier chargement des points de controles
	  Bezier *courbeB = new Bezier();
	  string attrName="P0x";
	  double Px;
	  double Py;
	  Point3D *ctrlPoint;
	  int numPoint = 0;
	  while(attr.hasAttribute(attrName))
	    {
	      Px = atof(attr.attribute( attrName, "" ));
	      char snumPoint[256]="";
	      sprintf(snumPoint,"%d",numPoint);
	      string temp = (string)snumPoint;
	      attrName= "P"+temp+"y";
	      Py = atof(attr.attribute( attrName, "" ));
	      ctrlPoint = new  Point3D(Px, Py, 0.0);
	      courbeB->addPoint(ctrlPoint);
	      numPoint++;
	      char snumPoint2[256]="";
	      sprintf(snumPoint2,"%d",numPoint);
	      string temp2 = (string)snumPoint2;
	      attrName="P"+temp2+"x";
	    }    
	  
	  p=courbeB;
	} 

      if(attr.tagName() == "bspline" )
	{//cration d'une courbe de Bspline chargement des points de controles
	  Bspline *courbeB = new Bspline();
	  string attrName="P0x";
	  double Px;
	  double Py;
	  Point3D *ctrlPoint;
	  int numPoint = 0;
	  while(attr.hasAttribute(attrName))
	    {
	      Px = atof(attr.attribute( attrName, "" ));
	      char snumPoint[256]="";
	      sprintf(snumPoint,"%d",numPoint);
	      string temp = (string)snumPoint;
	      attrName= "P"+temp+"y";
	      Py = atof(attr.attribute( attrName, "" ));
	      ctrlPoint = new  Point3D(Px, Py, 0.0);
	      courbeB->addPoint(ctrlPoint);
	      numPoint++;
	      char snumPoint2[256]="";
	      sprintf(snumPoint2,"%d",numPoint);
	      string temp2 = (string)snumPoint2;
	      attrName="P"+temp2+"x";
	    }    
	  
	  p=courbeB;
	} 

    }
    return p;
}

