/** @file modelml.h */

//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                       /
// Projet : Gnrateur de cylindres gnraliss                                          /
//                                                                                       /
// Nom du fichier : modelml.h                                                            /
//                                                                                       /
// Creation : 20/11/2005                                                                 /
//                                                                                       /
// Description : La classe Modelml regroupe l'ensemble des fonctionnalits ncssaires   /
//              au chargement et  la sauvegarde de fichier aux format xml. Ce parsing   /
//              XML utilise la Class QT QXML et l'approche DOM (Document Object Model).  /
//                                                                                       /            
// Auteurs : - Guitteny Fabrice                                                          /
//           - Idiart Baptiste                                                           /
//           - Le Goff Erwan                                                             /
//                                                                                       /
//////////////////////////////////////////////////////////////////////////////////////////

#ifndef MODELML_H
#define MODELML_H

#include <stdlib.h>
#include <qdom.h>
#include <qfile.h> 
#include <qapplication.h>

#include "../primitives/segment.h"
#include "../primitives/circle.h"
#include "../primitives/bezier.h"
#include "../primitives/bspline.h"
#include "../primitives/point.h"

/**
 * La classe Modelml regroupe l'ensemble des fonctionnalits ncssaires
 * au chargement et  la sauvegarde de fichier aux format xml. Ce parsing
 * XML utilise la Class QT QXML et l'approche DOM (Document Object Model). 
 *
 *  @author Guitteny Fabrice
 *  @author Idiart Baptiste
 *  @author Le Goff Erwan
 */

///////////////////////////////////////////////////////////////////////////////////////////
//  Classe Modelml                                                                       //
///////////////////////////////////////////////////////////////////////////////////////////

class Modelml : public QDomDocument
{
 private:
  /** ajout d'une balise
  * @param ens_prim ensemble de primtives d'une de la fenetre concerne
  * @param elem balise tudie
  */
  void appendElem(const vector<Primitive*> &ens_prim, QDomElement &elem);

  /** chargement d'une balise
  * @param e balise tudie
  * @param ens_res ensemble de primtives d'une de la fenetre concerne
  */
  void loadElem(QDomElement &e,vector<Primitive*> &ens_res);

  /** chargement d'un attribut
  * @param att lment tudi
  * @return cration d'un objet Primitive @see primitive.h
  */
  Primitive* loadAttribut(const QDomElement& att);

  
 public: 

  /** Constructeur de Modelml
  *  L'entete du document sera de type <!DOCTYPE ModelMG>
  */
  Modelml();

   /** sauvegarde au format Xml
   * @param output nom du ficher de sortie
   * @param primProfile ensemble des primitves de la fentre profil
   * @param primSection ensemble des primitves de la fentre section
   * @param primAxis ensemble des primitves de la fentre axe
   */
  void save(const string &output ,const vector<Primitive*> &primProfile,const vector<Primitive*> &primSection,const vector<Primitive*> &primAxis);

   /** chargement d'un fichier Xml
   * @param input fichier en lecture
   * @return vecteur de dimmension 3 contenant la liste des primitives pour chacune des fentres
   */
  vector<vector<Primitive*> > load(const string input);

};

#endif

